PROGRAM FUND21;

(*  Climate Framework for Uncertainty, Negotiation and Distribution       *)
(*  Version 2.10  -  4 September 2000                                     *)
(*  Richard S.J. Tol                                                      *)
(*  Institute for Environmental Studies, Vrije Universiteit, Amsterdam    *)

(*  The Climate FUND is an integrated climate-economy model to calculate  *)
(*  the socio-economic impacts of both climate change and greenhouse gas  *)
(*  emission reduction, and to weight these two against one another in    *)
(*  order to determine the optimal abatement strategy in several game     *)
(*  theoretic settings. FUND is a highly parameterised, scenario-driven,  *)
(*  closed-loop model of the economies of nine major world regions and    *)
(*  their interactions with the climate. The framework includes a range   *)
(*  of climate change impacts, including intangible damages and forced    *)
(*  migration, and a suite of emission reduction measures, including      *)
(*  interregional capital transfers. The development of the Climate FUND  *)
(*  is sponsored by the Dutch National Research Programme: Global Air     *)
(*  Pollution and Climate Change, the EU DG Research Fourth Framework     *)
(*  Programme, the National Science Foundation, and the Michael Otto      *)
(*  Foundation.                                                           *)

USES
    Crt, RP210, GenRand;

CONST
     scratch = true;                  (* start with no measures ? *)
     CBA = false;                     (* cost-benefit analysis ? *)
     CEA = false;                      (* cost-effectiveness analysis ? *)
     captrans = false;                (* capital transfers ? *)
     cooperate = false;               (* cooperate ? *)
     repeated = false;
     uncertain = false;               (* uncertainty ? *)
     sequent = true;                 (* sequential ? *)
     scengmt = false;                 (* scenario global mean temperature ? *)
     baseincl = false;
     kantian = false;
     compens = false;

     consleak = 0.25;                 (* consumption leak *)
     compth = 100;                    (* compensation threshold *)

     itmax = 25;

     startper = 1;
     noper = 20;
     repper = 20;

     riskave = 1;
     ineqave = 0;

     ampl = 4;
     altruism = 0.00; (* altruism should be zero if cooperation *)

     runs = 1;

     bforst = 1.00;
     btaxst = 0.0001;

     CBaU = 1792.3352;
     CMax = 550;

TYPE
    measure = regarr;
    matmeas = ARRAY [1..NoReg,1..NOI] OF double;
    regper = ARRAY [1..NoReg,1..NoPer] OF double;
    boolmea = ARRAY [1..NoReg,1..NOI] OF boolean;

VAR
   boolmeas: boolmea;
   regstep, forstep, taxstep: double;
   c2000,c2025,c2050,c2075,c2100,c2125,c2150,c2175,c2200,toptper,globemm,globcum: double;
   optper: word;
   globviol: boolean;
   emmrec, count: regarr;
   coop: regper;

   spbmbg, spbmsd, voslbg, voslsd, valincbg, valincsd, valbase: double;
   wrelbg, wrelsd, wrnlbg, wrnlsd, agelbg, agelsd, agnlbg, agnlsd, forel, fornl: double;
   heelbg, heelsd, henlbg, henlsd, ceelbg, ceelsd, cenlbg, cenlsd: double;
   dfelbg, dfelsd, dfnlbg, dfnlsd, smelbg, smelsd, smnlbg, smnlsd, malelbg, malelsd, malnlbg, malnlsd: double;
   vbinc, vbiaddbg, vbiaddsd: double;
   dvbmbg, dvbmsd, drydelbg, drydelsd, incdens, wvbmbg, wvbmsd, wetdelbg, wetdelsd: double;
   immcstbg, immcstsd, entertimebg, entertimesd, emcstbg, emcstsd, leavedelbg, leavedelsd: double;
   pluselbg, pluselsd, cvlinbg, cvlinsd, rlinbg, rlinsd, maxcardvascbg, maxcardvascsd: double;
   addmp: double;

FUNCTION Log(x: double): double;
(* returns ln(x) *)

BEGIN
     IF (x > 1.0E-308) AND (x < 1.0E308) THEN
        log:= ln(x)
     ELSE IF x < 1.0E-308 THEN
          log:= ln(1.0E-308)
     ELSE
         log:= ln(1.0E38)
END;


FUNCTION Expo(x: double): double;
(* returns exp(x) *)

BEGIN
     IF (x < 709) AND (x > -11356) THEN
        expo:= exp(x)
     ELSE IF x < -11356 THEN
          expo:= exp(-11356)
     ELSE
         expo:= exp(709)
END;

FUNCTION Root(x: double): double;
(* returns sqrt(x) *)

BEGIN
     IF x < 1.0E308 THEN
        root:= sqrt(x)
     ELSE
         root:= 1.0E154
END;

FUNCTION Quad(x: double): double;
(* returns x^2 *)

BEGIN
     IF x < 1.0E154 THEN
        quad:= sqr(x)
     ELSE
         quad:= 1.0E308
END;

FUNCTION Pow(x,y: double): double;
(* returns x^y *)

BEGIN
     pow:= expo(y*log(x))
END;

FUNCTION Interact(M,N: double): double;
(* interaction radiative forcing methane and nitrous oxide (in ppbv) *)

VAR hulp: double;

BEGIN
     hulp:= 1 + pow(M*N,0.75)*2.01E-5 + pow(M*N,1.52)*M*5.31E-15;
     interact:= 0.47*log(hulp)
END;

PROCEDURE InitImpactPar;

(* CODE
   sp = species
   for = forestry
   ag = agriculture
   he = heating energy
   ce = cooling energy
   wr = water resources
   df = dengue fever
   mal = malaria
   sm = schistosomiasis
   ccp = cardiovascular cold 65 plus
   ccm = cardiovascular cold 65 minus
   chp = cardiovascular heat 65 plus
   chm = cardiovascular heat 65 minus
   rp = respiratory 65 plus
   rm = respiratory 65 minus

   bg = best guess
   sd = standard deviation
   bm = benchmark
   nl = non-linearity
   el = elasticity
   del = delay
   l = linear
   q = quadratic
*)

VAR i,j: word;

BEGIN
     valbase:= 15965.86;
     valincbg:= 20000;
     valincsd:= 10000;
  (* 20000 is benchmark income; 15965.86 is OECD income in 1990 *)
     spbmbg:= 50;
     spbmsd:= 50;
     dvbmbg:= 4;
     dvbmsd:= 2;
     drydelbg:= 0;
     drydelsd:= 0.1;
     incdens:= 1196.099;
     (* 1196.099 is income density in 1990 *)
     wvbmbg:= 5;
     wvbmsd:= 2.5;
     wetdelbg:= 0;
     wetdelsd:= 0.1;
     immcstbg:= 0.4;
     immcstsd:= 0.2;
     entertimebg:= 3;
     entertimesd:= 1;
     emcstbg:= 3;
     emcstsd:= 1.5;
     leavedelbg:= 0;
     leavedelsd:= 0.1;
     wrelbg:= 0.85;
     wrelsd:= 0.15;
     wrnlbg:= 1;
     wrnlsd:= 0.5;
     forel:= -0.31;
     fornl:= 1;
     heelbg:= 0.8;
     heelsd:= 0.2;
     henlbg:= 1;
     henlsd:= 0.5;
     ceelbg:= 0.8;
     ceelsd:= 0.2;
     cenlbg:= 1;
     cenlsd:= 0.5;
     dfelbg:= 1.0;
     dfelsd:= 0.5;
     dfnlbg:= 1.0;
     dfnlsd:= 0.5;
     smelbg:= 1.0;
     smelsd:= 0.5;
     smnlbg:= 1.0;
     smnlsd:= 0.5;
     malelbg:= 1.0;
     malelsd:= 0.5;
     malnlbg:= 1.0;
     malnlsd:= 0.5;
     vbinc:= 3100;
     vbiaddbg:= 0;
     vbiaddsd:= 10;
     agnlbg:= 2;
     agnlsd:= 0.5;
     agelbg:= 0.31;
     agelsd:= 0.15;
     pluselbg:= 0.25;
     pluselsd:= 0.08;
     cvlinbg:= 0.025901;
     cvlinsd:= 0.009574;
     rlinbg:= 0.001583;
     rlinsd:= 0.000457;
     maxcardvascbg:= 0.05;
     maxcardvascsd:= 0.02;

     voslbg:= 200;
     voslsd:= 100;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              spbm:= spbmbg;
              valinc:= valincbg;
              valadj:= valbase/valinc/(1+valbase/valinc);
              vosl:= voslbg;
              wrbm:= wrbmbg;
              wrel:= wrelbg;
              wrnl:= wrnlbg;
              forbm:= forbmbg;
              hebm:= hebmbg;
              heel:= heelbg;
              henl:= henlbg;
              cebm:= cebmbg;
              ceel:= ceelbg;
              cenl:= cenlbg;
              dfbm:= dfbmbg;
              dfch:= dfchbg;
              dfel:= dfelbg;
              dfnl:= dfnlbg;
              smbm:= smbmbg;
              smch:= smchbg;
              smel:= smelbg;
              smnl:= smnlbg;
              malbm:= malbmbg;
              malch:= malchbg;
              malel:= malelbg;
              malnl:= malnlbg;
              vbiadd:= vbiaddbg;
              agrbm:= agrbmbg;
              aglbm:= aglbmbg;
              agopt:= agoptbg;
              agtime:= agtimebg;
              agdel:= 1-1/agtime;
              agnl:= agnlbg;
              agel:= agelbg;
              ccplbm:= ccplbmbg;
              ccpqbm:= ccpqbmbg;
              ccmlbm:= ccmlbmbg;
              ccmqbm:= ccmqbmbg;
              chplbm:= chplbmbg;
              chpqbm:= chpqbmbg;
              chmlbm:= chmlbmbg;
              chmqbm:= chmqbmbg;
              cvlin:= cvlinbg;
              plusel:= pluselbg;
              maxcardvasc:= maxcardvascbg;
              rlin:= rlinbg;
              rlbm:= rlbmbg;
              rqbm:= rqbmbg;
              wlbm:= wlbmbg;
              wvbm:= wvbmbg;
              wetdel:= wetdelbg;
              wmbm:= wmbmbg;
              dlbm:= dlbmbg;
              dvbm:= dvbmbg;
              drydel:= drydelbg;
              pc:= pcbg;
              emcst:= emcstbg;
              immcst:= immcstbg;
              leavedel:= leavedelbg;
              enterdel:= 1-1/entertimebg;
              FOR j:= 1 TO NoReg DO
                  immigrate[j]:= immigratebg[j]
         END
END;

PROCEDURE InitRegMult;

VAR i: word;

BEGIN
     economy[1].regmp:= 4.4788/2.5;
     economy[2].regmp:= 4.1401/2.5;
     economy[3].regmp:= 3.0492/2.5;
     economy[4].regmp:= 4.4397/2.5;
     economy[5].regmp:= 4.6320/2.5;
     economy[6].regmp:= 3.5218/2.5;
     economy[7].regmp:= 3.4416/2.5;
     economy[8].regmp:= 3.9997/2.5;
     economy[9].regmp:= 3.9804/2.5;

     FOR i:= 1 TO NoReg DO
         economy[i].regmp:= 1;

     FOR i:= 1 TO NoReg DO
         economy[i].regsq:= economy[i].regmp*economy[i].regmp
END;

PROCEDURE InitCoalition;

VAR i,j: word;

BEGIN
     coop[1,1]:= 0;
     coop[1,2]:= 0;
     coop[1,3]:= 0;
     coop[1,4]:= 0;
     coop[1,5]:= 0;
     coop[1,6]:= 0;
     coop[1,7]:= 0;
     coop[1,8]:= 0;
     coop[1,9]:= 0;
     coop[1,10]:= 0;
     coop[1,11]:= 0;
     coop[1,12]:= 0;
     coop[1,13]:= 0;
     coop[1,14]:= 0;
     coop[1,15]:= 0;
     coop[1,16]:= 0;
     coop[1,17]:= 0;
     coop[1,18]:= 0;
     coop[1,19]:= 0;
     coop[1,20]:= 0;
     coop[2,1]:= 0;
     coop[2,2]:= 0;
     coop[2,3]:= 0;
     coop[2,4]:= 0;
     coop[2,5]:= 1;
     coop[2,6]:= 1;
     coop[2,7]:= 1;
     coop[2,8]:= 1;
     coop[2,9]:= 1;
     coop[2,10]:= 1;
     coop[2,11]:= 1;
     coop[2,12]:= 1;
     coop[2,13]:= 1;
     coop[2,14]:= 1;
     coop[2,15]:= 1;
     coop[2,16]:= 1;
     coop[2,17]:= 1;
     coop[2,18]:= 1;
     coop[2,19]:= 1;
     coop[2,20]:= 1;
     coop[3,1]:= 0;
     coop[3,2]:= 0;
     coop[3,3]:= 0;
     coop[3,4]:= 0;
     coop[3,5]:= 0;
     coop[3,6]:= 0;
     coop[3,7]:= 0;
     coop[3,8]:= 0;
     coop[3,9]:= 0;
     coop[3,10]:= 0;
     coop[3,11]:= 0;
     coop[3,12]:= 0;
     coop[3,13]:= 0;
     coop[3,14]:= 0;
     coop[3,15]:= 0;
     coop[3,16]:= 0;
     coop[3,17]:= 0;
     coop[3,18]:= 0;
     coop[3,19]:= 0;
     coop[3,20]:= 0;
     coop[4,1]:= 1;
     coop[4,2]:= 1;
     coop[4,3]:= 1;
     coop[4,4]:= 1;
     coop[4,5]:= 1;
     coop[4,6]:= 1;
     coop[4,7]:= 1;
     coop[4,8]:= 1;
     coop[4,9]:= 1;
     coop[4,10]:= 1;
     coop[4,11]:= 1;
     coop[4,12]:= 1;
     coop[4,13]:= 1;
     coop[4,14]:= 1;
     coop[4,15]:= 1;
     coop[4,16]:= 1;
     coop[4,17]:= 1;
     coop[4,18]:= 1;
     coop[4,19]:= 1;
     coop[4,20]:= 1;
     coop[5,1]:= 0;
     coop[5,2]:= 0;
     coop[5,3]:= 0;
     coop[5,4]:= 0;
     coop[5,5]:= 0;
     coop[5,6]:= 0;
     coop[5,7]:= 0;
     coop[5,8]:= 0;
     coop[5,9]:= 0;
     coop[5,10]:= 0;
     coop[5,11]:= 0;
     coop[5,12]:= 0;
     coop[5,13]:= 0;
     coop[5,14]:= 0;
     coop[5,15]:= 0;
     coop[5,16]:= 0;
     coop[5,17]:= 0;
     coop[5,18]:= 0;
     coop[5,19]:= 0;
     coop[5,20]:= 0;
     coop[6,1]:= 0;
     coop[6,2]:= 0;
     coop[6,3]:= 0;
     coop[6,4]:= 0;
     coop[6,5]:= 0;
     coop[6,6]:= 0;
     coop[6,7]:= 0;
     coop[6,8]:= 0;
     coop[6,9]:= 1;
     coop[6,10]:= 1;
     coop[6,11]:= 1;
     coop[6,12]:= 1;
     coop[6,13]:= 1;
     coop[6,14]:= 1;
     coop[6,15]:= 1;
     coop[6,16]:= 1;
     coop[6,17]:= 1;
     coop[6,18]:= 1;
     coop[6,19]:= 1;
     coop[6,20]:= 1;
     coop[7,1]:= 0;
     coop[7,2]:= 0;
     coop[7,3]:= 0;
     coop[7,4]:= 0;
     coop[7,5]:= 0;
     coop[7,6]:= 0;
     coop[7,7]:= 1;
     coop[7,8]:= 1;
     coop[7,9]:= 1;
     coop[7,10]:= 1;
     coop[7,11]:= 1;
     coop[7,12]:= 1;
     coop[7,13]:= 1;
     coop[7,14]:= 1;
     coop[7,15]:= 1;
     coop[7,16]:= 1;
     coop[7,17]:= 1;
     coop[7,18]:= 1;
     coop[7,19]:= 1;
     coop[7,20]:= 1;
     coop[8,1]:= 0;
     coop[8,2]:= 0;
     coop[8,3]:= 0;
     coop[8,4]:= 0;
     coop[8,5]:= 0;
     coop[8,6]:= 0;
     coop[8,7]:= 0;
     coop[8,8]:= 0;
     coop[8,9]:= 0;
     coop[8,10]:= 0;
     coop[8,11]:= 0;
     coop[8,12]:= 0;
     coop[8,13]:= 0;
     coop[8,14]:= 0;
     coop[8,15]:= 0;
     coop[8,16]:= 0;
     coop[8,17]:= 0;
     coop[8,18]:= 0;
     coop[8,19]:= 0;
     coop[8,20]:= 0;
     coop[9,1]:= 1;
     coop[9,2]:= 1;
     coop[9,3]:= 1;
     coop[9,4]:= 1;
     coop[9,5]:= 1;
     coop[9,6]:= 1;
     coop[9,7]:= 1;
     coop[9,8]:= 1;
     coop[9,9]:= 1;
     coop[9,10]:= 1;
     coop[9,11]:= 1;
     coop[9,12]:= 1;
     coop[9,13]:= 1;
     coop[9,14]:= 1;
     coop[9,15]:= 1;
     coop[9,16]:= 1;
     coop[9,17]:= 1;
     coop[9,18]:= 1;
     coop[9,19]:= 1;
     coop[9,20]:= 1;

     FOR i:= 1 TO 9 DO
         FOR j:= 1 TO 20 DO
             coop[i,j]:= 0
END;

PROCEDURE Initialise;

VAR i, j: word;
    fil: text;

BEGIN
     addmp:= 1;
     InitRegMult;
     optper:= startper;
     C2200:= CBaU;
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              oda:= 0;
              new(ypcgrowth);
              new(pgrowth);
              new(urbanp);
              new(aeei);
              new(acei);
              new(forestemm)
         END;
     WITH Climate DO BEGIN
          decay[1]:= 1;
          decay[2]:= exp(-1/lifeco2);
          decay[3]:= exp(-1/lifeco3);
          decay[4]:= exp(-1/lifeco4);
          decay[5]:= exp(-1/lifeco5);
          frac[1]:= 0.13;
          frac[2]:= 0.20;
          frac[3]:= 0.32;
          frac[4]:= 0.25;
          frac[5]:= 0.10;
          ch4decay:= ch4dec;
          n2odecay:= n2odec;
          ch4sd:= 6.8346;
          n2osd:= 0.0059;
          new(addco2);
          new(addch4);
          new(addn2o);
(*          new(scentemp); *)
          new(ch4em);
          new(n2oem);
          new(acco2);
          new(acch4);
          new(acn2o)
     END;

     Economy[1].name:= 'OECD-A';
     Economy[2].name:= 'OECD-E';
     Economy[3].name:= 'OECD-P';
     Economy[4].name:= 'CEE&SU';
     Economy[5].name:= 'M-E   ';
     Economy[6].name:= 'L-A   ';
     Economy[7].name:= 'S&SEA ';
     Economy[8].name:= 'CPA   ';
     Economy[9].name:= 'AFR   ';

     Economy[1].captrans:= 0;
     Economy[2].captrans:= 0;
     Economy[3].captrans:= 0;
     Economy[4].captrans:= 0;
     Economy[5].captrans:= 0;
     Economy[6].captrans:= 0;
     Economy[7].captrans:= 0;
     Economy[8].captrans:= 0;
     Economy[9].captrans:= 0;

     Economy[1].ypccorr:= 0.421855;
     Economy[2].ypccorr:= 0.263729;
     Economy[3].ypccorr:= 0.244423;
     Economy[4].ypccorr:= 0.306673;
     Economy[5].ypccorr:= 0.373133;
     Economy[6].ypccorr:= 0.437524;
     Economy[7].ypccorr:= 0.196623;
     Economy[8].ypccorr:= 0.121762;
     Economy[9].ypccorr:= 0.724458;

     Economy[1].income90:= 5122.833;
     Economy[2].income90:= 5451.268;
     Economy[3].income90:= 3137.484;
     Economy[4].income90:= 2910.337;
     Economy[5].income90:=  368.936;
     Economy[6].income90:=  780.083;
     Economy[7].income90:=  790.004;
     Economy[8].income90:=  456.775;
     Economy[9].income90:=  389.387;

     Economy[1].population90:=  276.630;
     Economy[2].population90:=  438.152;
     Economy[3].population90:=  144.024;
     Economy[4].population90:=  405.013;
     Economy[5].population90:=  144.737;
     Economy[6].population90:=  436.048;
     Economy[7].population90:= 1534.476;
     Economy[8].population90:= 1256.053;
     Economy[9].population90:=  615.549;

     Economy[1].ypc90:= 18518.69;
     Economy[2].ypc90:= 12441.5;
     Economy[3].ypc90:= 21784.44;
     Economy[4].ypc90:= 7185.779;
     Economy[5].ypc90:= 2549.007;
     Economy[6].ypc90:= 1788.982;
     Economy[7].ypc90:= 514.8362;
     Economy[8].ypc90:= 363.6587;
     Economy[9].ypc90:= 632.5856;

     Economy[1].plus90:= 0.1246;
     Economy[2].plus90:= 0.1306;
     Economy[3].plus90:= 0.1165;
     Economy[4].plus90:= 0.0964;
     Economy[5].plus90:= 0.0304;
     Economy[6].plus90:= 0.1483;
     Economy[7].plus90:= 0.0452;
     Economy[8].plus90:= 0.0563;
     Economy[9].plus90:= 0.0575;

     Economy[1].cardvasc90:= 0.003979;
     Economy[2].cardvasc90:= 0.003979;
     Economy[3].cardvasc90:= 0.003979;
     Economy[4].cardvasc90:= 0.005986;
     Economy[5].cardvasc90:= 0.002575;
     Economy[6].cardvasc90:= 0.001777;
     Economy[7].cardvasc90:= 0.002358;
     Economy[8].cardvasc90:= 0.002265;
     Economy[9].cardvasc90:= 0.001795;

     Economy[1].resp90:= 0.000430;
     Economy[2].resp90:= 0.000430;
     Economy[3].resp90:= 0.000430;
     Economy[4].resp90:= 0.000457;
     Economy[5].resp90:= 0.000324;
     Economy[6].resp90:= 0.000261;
     Economy[7].resp90:= 0.000271;
     Economy[8].resp90:= 0.001349;
     Economy[9].resp90:= 0.000394;

     Economy[1].aeeicorr:= 1.347647;
     Economy[2].aeeicorr:= 1.451926;
     Economy[3].aeeicorr:= 1.029493;
     Economy[4].aeeicorr:= 1.094628;
     Economy[5].aeeicorr:= 0.377157;
     Economy[6].aeeicorr:= 1.020566;
     Economy[7].aeeicorr:= 0.850389;
     Economy[8].aeeicorr:= 0.672361;
     Economy[9].aeeicorr:= 0.763828;

     Economy[1].popdens90:=  28;
     Economy[2].popdens90:= 429;
     Economy[3].popdens90:= 256;
     Economy[4].popdens90:=  54;
     Economy[5].popdens90:=  79;
     Economy[6].popdens90:=  70;
     Economy[7].popdens90:= 364;
     Economy[8].popdens90:= 232;
     Economy[9].popdens90:= 267;

     Economy[1].trade[0]:= 0;
     Economy[1].trade[1]:= 0;
     Economy[1].trade[2]:= 0;
     Economy[1].trade[3]:= 0;
     Economy[1].trade[4]:= 0;
     Economy[1].trade[5]:= 0;
     Economy[1].trade[6]:= 0;
     Economy[1].trade[7]:= 0;
     Economy[1].trade[8]:= 0;
     Economy[1].trade[9]:= 0;
     Economy[1].trade[10]:= 0;
     Economy[1].trade[11]:= 0;
     Economy[1].trade[12]:= 0;
     Economy[1].trade[13]:= 0;
     Economy[1].trade[14]:= 0;
     Economy[1].trade[15]:= 0;
     Economy[1].trade[16]:= 0;
     Economy[1].trade[17]:= 0;
     Economy[1].trade[18]:= 0;
     Economy[1].trade[19]:= 0;
     Economy[1].trade[20]:= 0;
     Economy[2].trade[0]:= 0;
     Economy[2].trade[1]:= 0;
     Economy[2].trade[2]:= 0;
     Economy[2].trade[3]:= 0;
     Economy[2].trade[4]:= 0;
     Economy[2].trade[5]:= 0;
     Economy[2].trade[6]:= 0;
     Economy[2].trade[7]:= 0;
     Economy[2].trade[8]:= 0;
     Economy[2].trade[9]:= 0;
     Economy[2].trade[10]:= 0;
     Economy[2].trade[11]:= 0;
     Economy[2].trade[12]:= 0;
     Economy[2].trade[13]:= 0;
     Economy[2].trade[14]:= 0;
     Economy[2].trade[15]:= 0;
     Economy[2].trade[16]:= 0;
     Economy[2].trade[17]:= 0;
     Economy[2].trade[18]:= 0;
     Economy[2].trade[19]:= 0;
     Economy[2].trade[20]:= 0;
     Economy[3].trade[0]:= 0;
     Economy[3].trade[1]:= 0;
     Economy[3].trade[2]:= 0;
     Economy[3].trade[3]:= 0;
     Economy[3].trade[4]:= 0;
     Economy[3].trade[5]:= 0;
     Economy[3].trade[6]:= 0;
     Economy[3].trade[7]:= 0;
     Economy[3].trade[8]:= 0;
     Economy[3].trade[9]:= 0;
     Economy[3].trade[10]:= 0;
     Economy[3].trade[11]:= 0;
     Economy[3].trade[12]:= 0;
     Economy[3].trade[13]:= 0;
     Economy[3].trade[14]:= 0;
     Economy[3].trade[15]:= 0;
     Economy[3].trade[16]:= 0;
     Economy[3].trade[17]:= 0;
     Economy[3].trade[18]:= 0;
     Economy[3].trade[19]:= 0;
     Economy[3].trade[20]:= 0;
     Economy[4].trade[0]:= 0;
     Economy[4].trade[1]:= 4.62;      (* 2 BRM *)
     Economy[4].trade[2]:= 3.69;
     Economy[4].trade[3]:= 2.77;
     Economy[4].trade[4]:= 1.85;
     Economy[4].trade[5]:= 0.92;
     Economy[4].trade[6]:= 0;
     Economy[4].trade[7]:= 0;
     Economy[4].trade[8]:= 0;
     Economy[4].trade[9]:= 0;
     Economy[4].trade[10]:= 0;
     Economy[4].trade[11]:= 0;
     Economy[4].trade[12]:= 0;
     Economy[4].trade[13]:= 0;
     Economy[4].trade[14]:= 0;
     Economy[4].trade[15]:= 0;
     Economy[4].trade[16]:= 0;
     Economy[4].trade[17]:= 0;
     Economy[4].trade[18]:= 0;
     Economy[4].trade[19]:= 0;
     Economy[4].trade[20]:= 0;
     Economy[5].trade[0]:= 0;
     Economy[5].trade[1]:= 2.63;           (* 10 BRM *)
     Economy[5].trade[2]:= 2.10;
     Economy[5].trade[3]:= 1.58;
     Economy[5].trade[4]:= 1.05;
     Economy[5].trade[5]:= 0.53;
     Economy[5].trade[6]:= 0;
     Economy[5].trade[7]:= 0;
     Economy[5].trade[8]:= 0;
     Economy[5].trade[9]:= 0;
     Economy[5].trade[10]:= 0;
     Economy[5].trade[11]:= 0;
     Economy[5].trade[12]:= 0;
     Economy[5].trade[13]:= 0;
     Economy[5].trade[14]:= 0;
     Economy[5].trade[15]:= 0;
     Economy[5].trade[16]:= 0;
     Economy[5].trade[17]:= 0;
     Economy[5].trade[18]:= 0;
     Economy[5].trade[19]:= 0;
     Economy[5].trade[20]:= 0;
     Economy[6].trade[0]:= 0;
     Economy[6].trade[1]:= 0.53;        (* 0.8 BRM *)
     Economy[6].trade[2]:= 0.43;
     Economy[6].trade[3]:= 0.32;
     Economy[6].trade[4]:= 0.21;
     Economy[6].trade[5]:= 0.11;
     Economy[6].trade[6]:= 0;
     Economy[6].trade[7]:= 0;
     Economy[6].trade[8]:= 0;
     Economy[6].trade[9]:= 0;
     Economy[6].trade[10]:= 0;
     Economy[6].trade[11]:= 0;
     Economy[6].trade[12]:= 0;
     Economy[6].trade[13]:= 0;
     Economy[6].trade[14]:= 0;
     Economy[6].trade[15]:= 0;
     Economy[6].trade[16]:= 0;
     Economy[6].trade[17]:= 0;
     Economy[6].trade[18]:= 0;
     Economy[6].trade[19]:= 0;
     Economy[6].trade[20]:= 0;
     Economy[7].trade[0]:= 0;
     Economy[7].trade[1]:= 0.46;     (* 0.5 BRM *)
     Economy[7].trade[2]:= 0.37;
     Economy[7].trade[3]:= 0.28;
     Economy[7].trade[4]:= 0.19;
     Economy[7].trade[5]:= 0.09;
     Economy[7].trade[6]:= 0;
     Economy[7].trade[7]:= 0;
     Economy[7].trade[8]:= 0;
     Economy[7].trade[9]:= 0;
     Economy[7].trade[10]:= 0;
     Economy[7].trade[11]:= 0;
     Economy[7].trade[12]:= 0;
     Economy[7].trade[13]:= 0;
     Economy[7].trade[14]:= 0;
     Economy[7].trade[15]:= 0;
     Economy[7].trade[16]:= 0;
     Economy[7].trade[17]:= 0;
     Economy[7].trade[18]:= 0;
     Economy[7].trade[19]:= 0;
     Economy[7].trade[20]:= 0;
     Economy[8].trade[0]:= 0;
     Economy[8].trade[1]:= 0.53;     (* 1.3 BRM *)
     Economy[8].trade[2]:= 0.42;
     Economy[8].trade[3]:= 0.32;
     Economy[8].trade[4]:= 0.21;
     Economy[8].trade[5]:= 0.11;
     Economy[8].trade[6]:= 0;
     Economy[8].trade[7]:= 0;
     Economy[8].trade[8]:= 0;
     Economy[8].trade[9]:= 0;
     Economy[8].trade[10]:= 0;
     Economy[8].trade[11]:= 0;
     Economy[8].trade[12]:= 0;
     Economy[8].trade[13]:= 0;
     Economy[8].trade[14]:= 0;
     Economy[8].trade[15]:= 0;
     Economy[8].trade[16]:= 0;
     Economy[8].trade[17]:= 0;
     Economy[8].trade[18]:= 0;
     Economy[8].trade[19]:= 0;
     Economy[8].trade[20]:= 0;
     Economy[9].trade[0]:= 0;
     Economy[9].trade[1]:= 1.08;   (* 2.9 BRM *)
     Economy[9].trade[2]:= 0.86;
     Economy[9].trade[3]:= 0.65;
     Economy[9].trade[4]:= 0.43;
     Economy[9].trade[5]:= 0.22;
     Economy[9].trade[6]:= 0;
     Economy[9].trade[7]:= 0;
     Economy[9].trade[8]:= 0;
     Economy[9].trade[9]:= 0;
     Economy[9].trade[10]:= 0;
     Economy[9].trade[11]:= 0;
     Economy[9].trade[12]:= 0;
     Economy[9].trade[13]:= 0;
     Economy[9].trade[14]:= 0;
     Economy[9].trade[15]:= 0;
     Economy[9].trade[16]:= 0;
     Economy[9].trade[17]:= 0;
     Economy[9].trade[18]:= 0;
     Economy[9].trade[19]:= 0;
     Economy[9].trade[20]:= 0;

     ReadWater;
     ReadForestry;
     ReadHeating;
     ReadCooling;
     ReadDengue;
     ReadSchisto;
     ReadMalaria;
     ReadAgricult;
     ReadCardHeat;
     ReadCardCold;
     ReadResp;
     ReadYLL;
     ReadYLD;
     ReadWetland;
     ReadDryland;
     ReadProtect;
     ReadImmigrate;

     ReadIncome;
     ReadPopulation;
     ReadYpCGrowth;
     ReadAgrish;
     ReadPGrowth;
     ReadUrbanPop;
     ReadACEI;
     ReadAEEI;
     ReadForestEmm;
     ReadForParam;
     ReadDiscRate;
     ReadTaxPar;
     ReadAddCO2;
     ReadAddCH4;
     ReadAddN2O;
     ReadCH4Em;
     ReadN2OEm;
(*     ReadScenGMT; *)
     ReadNCEmmRestr;
     ReadCEmmRestr;
     ReadScenUnc;
     WITH climate DO BEGIN
          temps:= modet/6.3/ln(2);
          seas:= modes;
          delaysea:= 1/lifesea;
          delaytemp:= 1/lifetemp
     END;
     bstsq:= quad(baset);
     dbstsq:= quad(dbst);
     bsssq:= quad(bases);
     dbsssq:= quad(dbss);
     forstep:= bforst;
     taxstep:= btaxst;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              ecgradd:= 0;
              pgadd:= 0;
              aeeiadd:= 0;
              aceiadd:= 0;
              foremadd:= 0;
              taxmp:= 1;
              formp:= 1;
         END;
     ReadAEEI;
END;

PROCEDURE ReInit;

VAR i, j: word;
fil: text;

BEGIN
     economy[1].emissint:= (*0.2567013*) 0.283/0.5529; (*emf*)
     economy[2].emissint:= (*0.139178*)  0.220/0.4699;
     economy[3].emissint:= (*0.1144091*) 0.118/0.9341;
     economy[4].emissint:= (*0.4464261*) 0.413/0.8092;
     economy[5].emissint:= (*0.4752512*) 0.455/6.0675;
     economy[6].emissint:= (*0.3097415*) 0.349/0.9486;
     economy[7].emissint:= (*0.4460469*) 0.492/1.3236;
     economy[8].emissint:= (*1.6008936*) 1.732/1.6073;
     economy[9].emissint:= (*0.4555767*) 0.440/1.6773;

     economy[1].cummemm:= 24965.9;
     economy[2].cummemm:= 25854.1;
     economy[3].cummemm:=  1661.1;
     economy[4].cummemm:=  5413.3;
     economy[5].cummemm:=   150.3;
     economy[6].cummemm:=   980.2;
     economy[7].cummemm:=   793.4;
     economy[8].cummemm:=   775.5;
     economy[9].cummemm:=   472.4;
     globcum:= 60994.3;

     InitImpactPar;
     totalpop:= 0;
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              kant:= 0;
              inc0:= incbase + captrans;
              income:= inc0;
              population:= pop0;
              IF riskave = 1 THEN
                 welf0:= log(1000*(1-consleak)*inc0/pop0)
              ELSE
                  welf0:= pow(1000*(1-consleak)*inc0/pop0,1-riskave)/(1-riskave);
              welfare:= welf0;
              YpC:= 1000*inc0/pop0;                     (* ????????? *)
              YpC0:= YpC;
              energint:= 1;                             (* confirmed *)
              energuse:= inc0;
              ryg:= 0;
              reei:= 0;                                 (* confirmed *)
              rcei:= 0;                                 (* confirmed *)
              htax:= 0;
              emission:= emissint*energuse;
              emiss0:= emission;                        (* confirmed *)
              totalpop:= totalpop + pop0;
              cdr:= 1;
              y:= 1;
              v:= 1;
              p:= 1;
              pg:= pgrowth^[0]/100;

              agrir:= 0;
              wetmeas:= 0;
              drycost:= 0;
              wetcost:= 0;
              leave:= 0;
              cumleave:= 0;
              cumenter:= 0;
              entercost:= 0;
              leavecost:= 0;

              eloss:= 0;
              sloss:= 0
     END;
     WITH climate DO BEGIN
          ch4add:= 0;
          n2oadd:= 0;
          temp:= 0.20;
(*          scentemp^[0]:= temp; *)
          tsq:= 0;
          dt:= 0.004986;
          dtsq:= 0;
          sea:= 0;
          ssq:= 0;
          ds:= 0.001255;
          dssq:= 0;
          cbox[1]:= 283.53;
          cbox[2]:= 5.62;
          cbox[3]:= 6.29;
          cbox[4]:= 2.19;
          cbox[5]:= 0.15;
          acco2^[0]:= 0;
          acch4^[0]:= 1222;
          acn2o^[0]:= 296;
          FOR i:= 1 TO 5 DO
              acco2^[0]:= acco2^[0] + cbox[i];
          ch4n2o:= interact(ch4pre,n2opre)
     END
END;

(* TAX PROCEDURES *)

PROCEDURE SetTax1(i,j: word);
(* sets tax country I in decade J to zero *)

BEGIN
     economy[i].tax[j]:= 0;
END;

PROCEDURE SetTax2;
(* sets all taxes to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 0 TO CtrMax DO
             economy[i].tax[j]:= 0;
END;

PROCEDURE SetTax3(i: word; T: double);

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].tax[j]:= T
     ELSE
         economy[i].tax[optper]:= T;
END;

PROCEDURE SetTax4(T: measure);

VAR i,j: word;

BEGIN
     IF NOT sequent THEN
        FOR i:= 1 TO NoReg DO
            FOR j:= 0 TO CtrMax DO
                economy[i].tax[j]:= T[i]
     ELSE
         FOR i:= 1 TO NoReg DO
             economy[i].tax[optper]:= T[i]
END;

PROCEDURE SetTax5(i,j: word; T: double);

BEGIN
     economy[i].tax[j]:= T
END;

PROCEDURE ObjTax(i: word);
(* changes taxes and influences for economy I *)

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].tax[j]:= economy[i].tax[j] + taxstep
     ELSE
         economy[i].tax[optper]:= economy[i].tax[optper] + taxstep
END;

PROCEDURE TaxObj(i: word);
(* changes taxes and influences one taxstep back for economy I *)

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].tax[j]:= economy[i].tax[j] - taxstep
     ELSE
         economy[i].tax[optper]:= economy[i].tax[optper] - taxstep
END;

(* FORESTRY PROCEDURES *)

PROCEDURE SetInflFor(VAR country: regrec; t: word);
(* determines the influence of COUNTRY.FORESTRY on the economy of COUNTRY *)

VAR fca, fcd, fa, fd, th1, th2: double;

BEGIN
     WITH country DO BEGIN
          IF forestry[optper] > 0 THEN BEGIN
             th1:= forconst - forlinear + root(quad(forconst-forlinear)-4*forconst*forsquare);
             th1:= th1/2/forsquare;
             th2:= (3*forconst/2 - forlinear)/2/forsquare;
             IF forestry[optper] < (th1 + th2) THEN BEGIN
                fd:= forestry[optper];
                fa:= 0
             END
             ELSE IF forestry[optper] < (2*th1 + th2) THEN BEGIN
                  fd:= th1 + th2;
                  fa:= forestry[optper] - fd
             END
             ELSE BEGIN
                  fa:= th1;
                  fd:= forestry[optper] - fa
             END;
             IF fd > forestemm^[t] THEN BEGIN    (* !!! *)
                fd:= forestemm^[t];
                fa:= forestry[optper] - fd
             END;
             IF fd > 0 THEN BEGIN
                fcd:= 2*(forconst + forlinear*fd + forsquare*quad(fd))/3;
                IF fcd < forconst*fd THEN
                   fcd:= 2*forconst*fd/3
             END
             ELSE
                 fcd:= 0;
             IF fa > 0 THEN BEGIN
                fca:= forconst + forlinear*fa + forsquare*quad(fa);
                IF fca < forconst*fa THEN
                   fca:= forconst*fa
             END
             ELSE
                 fca:= 0;
             forcost:= fca + fcd
          END
          ELSE
              forcost:= 0
     END
END;

PROCEDURE SetFor1(i,j: word);
(* sets forestry in country I in decade J to zero *)

BEGIN
     economy[i].forestry[j]:= 0;
END;

PROCEDURE SetFor2;
(* sets all forestry to zero *)

VAR i, j: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         FOR j:= 1 TO CtrMax DO
             economy[i].forestry[j]:= 0;
END;

PROCEDURE SetFor3(i: word; T: double);

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].forestry[j]:= T
     ELSE
         economy[i].forestry[optper]:= T;
END;

PROCEDURE SetFor4(T: measure);

VAR i,j: word;

BEGIN
     IF NOT sequent THEN
        FOR i:= 1 TO NoReg DO
            FOR j:= 0 TO CtrMax DO
                economy[i].forestry[j]:= T[i]
     ELSE
         FOR i:= 1 TO NoReg DO
             economy[i].forestry[optper]:= T[i]
END;

PROCEDURE ObjFor(i: word);
(* changes forestry in economy I *)

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].forestry[j]:= economy[i].forestry[j] + forstep
     ELSE
         economy[i].forestry[optper]:= economy[i].forestry[optper] + forstep
END;

PROCEDURE ForObj(i: word);
(* changes forestry one forstep back for economy I *)

VAR j: word;

BEGIN
     IF NOT sequent THEN
        FOR j:= 0 TO CtrMax DO
            economy[i].forestry[j]:= economy[i].forestry[j] - forstep
     ELSE
         economy[i].forestry[optper]:= economy[i].forestry[optper] - forstep
END;

(* SETTINGS *)

PROCEDURE SetMeasures1;
(* sets all measures to zero *)

BEGIN
     SetTax2;
     SetFor2
END;

PROCEDURE SetMeasures2;

VAR i,j: word;

BEGIN
     FOR i:= 1 TO NoPer DO
         FOR j:= 1 TO NoReg DO
             economy[j].tax[i]:= random/100
END;

PROCEDURE SetMeasures3(w: word);

VAR s: string;
    inp: text;
    i,j: word;

BEGIN
     str(w,s);
     assign(inp,sessionpathname+'seq'+s+'.txt');
     reset(inp);
     FOR i:= 0 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].tax[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].forestry[i]);
         readln(inp)
     END;
     close(inp)
END;

PROCEDURE SetMeasures4;

VAR inp: text;
    i,j: word;

BEGIN
     assign(inp,sessionpathname+'cea.txt');
     reset(inp);
     FOR i:= 0 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].tax[i]);
         readln(inp);
         FOR j:= 1 TO NoReg DO
             read(inp,economy[j].forestry[i]);
         readln(inp)
     END;
     close(inp)
END;

FUNCTION CheckStop(w: word): boolean;

VAR s: string;
    inp0,inp1: text;
    i,j: word;
    h0,h1: double;
    stop: boolean;

BEGIN
     str(w-1,s);
     assign(inp0,sessionpathname+'seq'+s+'.txt');
     reset(inp0);
     str(w,s);
     assign(inp1,sessionpathname+'seq'+s+'.txt');
     reset(inp1);
     stop:= true;
     FOR i:= 0 TO NoPer DO BEGIN
         FOR j:= 1 TO NoReg DO BEGIN
             read(inp0,h0);
             read(inp1,h1);
             stop:= stop AND (ABS(h1-h0) < 0.5*btaxst)
         END;
         readln(inp0);
         readln(inp1);
         FOR j:= 1 TO NoReg DO BEGIN
             read(inp0,h0);
             read(inp1,h1);
             stop:= stop AND (ABS(h1-h0) < 0.5*bforst)
         END;
         readln(inp0);
         readln(inp1)
     END;
     close(inp0);
     close(inp1);
     checkstop:= stop
END;

(* CALCULATING PROCEDURES *)

PROCEDURE Calculate(VAR country: regrec; t: word);

VAR i, j, decade: word;
    s, h, henter, excessloss, popold, ic, dc, x, noemer, seef: double;
    ygrowth, urbpop: double;
    incth: boolean;

BEGIN
     WITH country, climate DO BEGIN
          IF t > 50 THEN BEGIN
             ygrowth:= (1+0.01*(pgrowth^[t-1]+pgadd*(t-50)))*(1+0.01*(ypcgrowth^[t-1]+ecgradd*(t-50))) - 1;
             population:= (1+0.01*(pgrowth^[t-1]+pgadd*(t-50)))*population;
             energint:= (1 - 0.01*(aeei^[t] + aeeiadd*(t-50)) - reei)*energint;
             emissint:= (1 - 0.01*(acei^[t] + aceiadd*(t-50)) - rcei)*emissint
          END
          ELSE BEGIN
               ygrowth:= (1+0.01*pgrowth^[t-1])*(1+0.01*ypcgrowth^[t-1]) - 1;
               population:= (1+0.01*pgrowth^[t-1])*population;
               energint:= (1 - 0.01*aeei^[t] - reei)*energint;
               emissint:= (1 - 0.01*acei^[t] - rcei)*emissint
          END;
          income:= (1 + ygrowth - taxmp*ryg)*income;
          IF (t > 50) AND (t <= 60) THEN
             income:= income + oda;
          y:= income/inc0;
          w1950:= welfare/welf0;
          popold:= population;
          p:= population/pop0;
          IF population < 0.000000001 THEN
             YpC:= 0
          ELSE
              YpC:= 1000*income/population;
          v:= ypc/ypc0;
          agrish:= agrish0*pow(v,-agel);
          energuse:= energint*income;
          emission:= emissint*energuse;
          IF income > 0 THEN BEGIN
             cummemm:= cummemm+emission;
             globcum:= globcum+emission
          END
          ELSE BEGIN
               globcum:= globcum-cummemm;
               cummemm:= 0
          END;
          IF (t mod 10 = 0) AND (t >= 40) THEN BEGIN
             decade:= trunc((t-40)/10);
             htax:= tax[decade];
             ryg:= (taxpar1*htax + taxpar2*quad(htax));
             reei:= 0.5*htax;
             rcei:= 0.5*htax;
(*             IF NOT ((name = 'OECD-A') OR (name = 'OECD-E') OR (name = 'OECD-P')) THEN BEGIN
                noemer:= economy[1].income+economy[2].income+economy[3].income;
                IF noemer > 0.0000001 THEN
                   ryg:= ryg + trade[decade]*(economy[1].ryg*economy[1].income+economy[2].ryg*economy[2].income
                         +economy[3].ryg*economy[3].income)/noemer
             END
*)          END;
          setinflfor(country,t);

          species:= spbm/valbase*ypc/valinc/(1+ypc/valinc)/valadj*ypc*population/1000;
          water:= wrbm*income90*pow(income/income90,wrel)*pow(temp,wrnl);
          forests:= forbm*income*pow(ypc/ypc90,forel)*pow(temp,fornl);
          heating:= hebm*income90*aeeicorr*pow(temp,henl)*pow(ypc/ypc90,heel)*population/population90;
          cooling:= cebm*income90*aeeicorr*pow(temp,cenl)*pow(ypc/ypc90,ceel)*population/population90;
          IF t > 50 THEN BEGIN
             hebm:= hebm*(1 - 0.01*(aeei^[t] + aeeiadd*(t-50)) - reei);
             cebm:= cebm*(1 - 0.01*(aeei^[t] + aeeiadd*(t-50)) - reei);
          END;
          IF t > 50 THEN
             incth:= (ypc < vbinc + vbiadd*(t-50))
          ELSE
              incth:= (ypc < vbinc);
          IF incth THEN
             IF baseincl THEN BEGIN
                dfbase:= dfbspar*population*pow((vbinc-ypc)/(vbinc-ypc90),dfel);
                smbase:= smbspar*population*pow((vbinc-ypc)/(vbinc-ypc90),smel);
                malbase:= malbspar*population*pow((vbinc-ypc)/(vbinc-ypc90),malel);
                dengue:= dfch*dfbase*pow(temp,dfnl);
                schisto:= smch*smbase*pow(temp,smnl);
                malaria:= malch*malbase*pow(temp,malnl)
             END
             ELSE BEGIN
                  dengue:= dfbm*population*pow(temp,dfnl)*pow((vbinc-ypc)/(vbinc-ypc90),dfel);
                  schisto:= smbm*population*pow(temp,smnl)*pow((vbinc-ypc)/(vbinc-ypc90),smel);
                  malaria:= malbm*population*pow(temp,malnl)*pow((vbinc-ypc)/(vbinc-ypc90),malel);
                  dfbase:= 0;
                  smbase:= 0;
                  malbase:= 0
             END
          ELSE BEGIN
               dengue:= 0;
               schisto:= 0;
               malaria:= 0;
               dfbase:= 0;
               smbase:= 0;
               malbase:= 0
          END;
          agrir:= agrbm*pow(dt/dbst,agnl) + agdel*agrir;
          IF agopt > 0.5 THEN
             agril:= (-2*aglbm*agopt*temp + aglbm*temp*temp)/(1-2*agopt)
          ELSE IF temp < agopt THEN
               agril:= aglbm*2*temp/agopt
          ELSE
              agril:= aglbm*(agopt-2+temp)/(agopt-1);
          agcost:= (agril+agrir)*agrish*income;
          plus:= plus90*pow(ypc/ypc90,plusel);
          basecardvasc:= (cardvasc90 + cvlin*(plus-plus90))*population;
          baseresp:= (resp90 + rlin*(plus-plus90))*population;

          cardheat:= (chplbm*plus + chmlbm*(1-plus))*temp +
                     (chpqbm*plus + chmqbm*(1-plus))*temp*temp;
          resp:= rlbm*temp + rqbm*temp*temp;
          IF t > 50 THEN BEGIN
             urbpop:= urbanp^[t]+urbanadd*(t-50);
             IF urbpop > 0.98 THEN
                urbpop:= 0.98;
             IF urbpop < 0.15 THEN
                urbpop:= 0.15;
             cardheat:= cardheat*urbpop*population*10;
             IF cardheat > 1000000*maxcardvasc*urbpop*basecardvasc THEN
                cardheat:= 1000000*maxcardvasc*urbpop*basecardvasc;
             resp:= resp*urbpop*population*10;
             IF resp > 1000000*maxcardvasc*urbpop*baseresp THEN
                resp:= 1000000*maxcardvasc*urbpop*baseresp;
          END
          ELSE BEGIN
             cardheat:= cardheat*urbanp^[t]*population*10;
             IF cardheat > 1000000*maxcardvasc*urbanp^[t]*basecardvasc THEN
                cardheat:= 1000000*maxcardvasc*urbanp^[t]*basecardvasc;
             resp:= resp*urbanp^[t]*population*10;
             IF resp > 1000000*maxcardvasc*urbanp^[t]*baseresp THEN
                resp:= 1000000*maxcardvasc*urbanp^[t]*baseresp;
          END;
          IF cardheat < 0 THEN
             cardheat:= 0;
          IF resp < 0 THEN
             resp:= 0;
          cardcold:= (ccplbm*plus + ccmlbm*(1-plus))*temp +
                     (ccpqbm*plus + ccmqbm*(1-plus))*temp*temp;
          cardcold:= cardcold*population*10;
          IF cardcold < -1000000*maxcardvasc*basecardvasc THEN
             cardcold:= -1000000*maxcardvasc*basecardvasc;
          IF cardcold > 0 THEN
             cardcold:= 0;

          dead:= dengue + schisto + malaria + cardheat + cardcold + resp;
          yll:= d2ld*dengue+d2ls*schisto+d2lm*schisto+d2lc*cardheat+d2lc*cardcold+d2lr*resp;
          yld:= d2dd*dengue+d2ds*schisto+d2dm*schisto+d2dc*cardheat+d2dc*cardcold+d2dr*resp;
          daly:= yll+yld;
          deadcost:= vosl*ypc*dead/1000000000;
(*          deadcost:= vosl*ypc*(dengue+malaria+cardheat+resp)/1000000000; *)
(*          deadcost:= vosl/20*yll*dead/1000000000; (* EXTERNE *)
(*          deadcost:= vosl/20*ypc*yll/1000000000;*) (* ????? *)

          popdens:= popdens90*population/population90;
          wetval:= wvbm/valbase*ypc/valinc/(1+ypc/valinc)/valadj*ypc;
          dryval:= dvbm*popdens*ypc/1000/incdens;
          npprotcost:= pc*ds*100*(1+discrate+ygrowth)/(discrate+ygrowth);
          IF npprotcost < 0 THEN
             npprotcost:= 0;
          npwetcost:= wetval*wmbm*ds/(discrate+ygrowth);
          IF npwetcost < 0 THEN
             npwetcost:= 0;
          npdrycost:= dryval*dlbm*ds*(1+ygrowth)*(discrate+ygrowth)/discrate/discrate;
          IF npdrycost < 0.00000001 THEN BEGIN
             npdrycost:= 0;
             protlev:= 0
          END
          ELSE
              protlev:= 1-0.5*(npprotcost+npwetcost)/npdrycost;
          IF protlev < 0 THEN
             protlev:= 0;
          wetland:= wlbm*sea;
          wetmeas:= wmbm*ds*protlev + wetmeas;
          wettot:= wetland + wetmeas;
          wetann:= (wlbm + wmbm*protlev)*ds;
          IF wettot > wetmax THEN BEGIN
             wettot:= wetmax;
             wetann:= 0
          END;
          wetcost:= wetval*wetann/1000 + wetdel*wetcost;
          dryland:= dlbm*sea*(1-protlev);
          drycost:= dryval*dlbm*ds*(1-protlev)/1000 + drydel*drycost;
          protcost:= protlev*pc*ds/10;
          enter:= 0;
          FOR j:= 1 TO NoReg DO
              enter:= enter + immigrate[j]*economy[j].leave;
          leave:= popdens*dlbm*ds*(1-protlev);
          cumleave:= leave + cumleave;
          cumenter:= enter + cumenter;
          leavecost:= emcst*ypc*leave/1000000000 + leavedel*leavecost;
          entercost:= immcst*ypc*enter/1000000000 + enterdel*entercost;

          eloss:= - water - forests - heating - cooling - agcost + drycost + protcost + entercost;
          sloss:= species + deadcost + wetcost + leavecost;
          income:= income - formp*forcost*y;
          IF t >= 40 THEN
             IF baseincl THEN
                population:= population + (enter - dead - leave - dfbase - malbase - smbase)/1000000
             ELSE
                 population:= population + (enter - dead - leave)/1000000;
          IF popold < 0.000000001 THEN
             pg:= 0
          ELSE
              pg:= (population-popold)/popold;
          welfare:= (1-consleak)*income - eloss;
          IF t >= 40 THEN
             income:= income - consleak*eloss;
(*          eloss:= drycost + protcost + entercost;
          IF water < 0 THEN
             eloss:= eloss - water;
          IF forests < 0 THEN
             eloss:= eloss - forests;
          IF heating < 0 THEN
             eloss:= eloss - heating;
          IF cooling < 0 THEN
             eloss:= eloss - cooling;
          IF agcost < 0 THEN
             eloss:= eloss - agcost; *)

          IF (population < 0.00000001) OR ((sloss+kant) >= welfare) OR (welfare <= 0) THEN
             welfare:= -100000000
          ELSE BEGIN
              welfare:= 1000*(welfare-sloss-kant)/population;
              IF riskave = 1 THEN
                 welfare:= log(welfare)
              ELSE
                  welfare:= pow(welfare,1-riskave)/(1-riskave)
          END;
          IF income < 0 THEN BEGIN
             income:= 0;
             eloss:= 0;
             sloss:= 0
          END
          ELSE BEGIN
               eloss:= eloss/income;
               sloss:= sloss/income
          END;
          IF t = 40+10*optper THEN BEGIN
             cdr:= 1;
             hNPVW:= welfare
          END;
          IF t > 40+10*optper THEN BEGIN
             cdr:= (1+discrate)*cdr;
             IF hNPVW >= 1.0E-309 THEN
                hNPVW:= hNPVW + welfare/cdr
          END;
          totalpop:= totalpop + population
     END
END;

PROCEDURE Write1(VAR country: regrec; t: word; VAR els,sls,
                     ent,mal,sch,den,car,res,lea,inc,wel,pop,mc: text);

BEGIN
     WITH country, climate DO BEGIN
          write(ent,enter:10:4,' ');
          write(lea,leave:10:4,' ');
          IF baseincl THEN BEGIN
             write(mal,malbase:10:4,' ',' ',malaria:10:4,' ');
             write(sch,smbase:10:4,' ',schisto:10:4,' ');
             write(den,dfbase:10:4,' ',dengue:10:4,' ');
          END
          ELSE BEGIN
               write(mal,malaria:10:4,' ');
               write(sch,schisto:10:4,' ');
               write(den,dengue:10:4,' ')
          END;
          write(car,cardheat+cardcold:10:4,' ');
          write(res,resp:10:4,' ');
          IF income > 0.000001 THEN BEGIN
             write(els,eloss*100:10:6,' ');
             write(sls,sloss*100:10:6,' ');
             IF t >= 40 THEN
                write(mc,2*taxpar2*tax[trunc((t-40)/10)]*income*(1+discrate)/discrate/emission:8:2,' ')
             ELSE
                 write(mc,0,' ');
          END
          ELSE BEGIN
               write(els,0,' ');
               write(sls,0,' ');
               write(mc,0,' ')
          END;
          write(inc,income:12:6,' ');
          write(wel,emission:12:6,' ');
          write(pop,population:12:6,' ');
     END
END;

PROCEDURE WriteM(VAR country: regrec; t: word; VAR els,sls,inc,pop: text);

BEGIN
     WITH country, climate DO BEGIN
          IF income > 0.000001 THEN BEGIN
             write(els,eloss*100:14:10,' ');
             write(sls,sloss*100:14:10,' ');
          END
          ELSE BEGIN
               write(els,0,' ');
               write(sls,0,' ');
          END;
          write(inc,income:12:6,' ');
          write(pop,population:12:6,' ');
     END
END;

PROCEDURE CalcClimate(j: word);
(* calculates the climatic variables at time J *)

VAR mco2, emfforest: double;
    i: word;
    ht, hdt: double;

BEGIN
     WITH climate DO BEGIN
          mco2:= 0;
          FOR i:= 1 TO NoReg DO
              WITH economy[i] DO BEGIN
                   mco2:= mco2 + emission + forestemm^[j] - forestry[trunc(j/10)]
              END;
          IF j > 50 THEN
             FOR i:= 1 TO NoReg DO
                 mco2:= mco2 + economy[i].foremadd*(j-50);
          FOR i:= 1 TO 5 DO
              cbox[i]:= cbox[i]*decay[i] + 0.000471*frac[i]*(mco2 + addmp*addco2^[j]);
          acco2^[j]:= 0;
          FOR i:= 1 TO 5 DO
              acco2^[j]:= acco2^[j] + cbox[i];
          IF j > 50 THEN BEGIN
             acch4^[j]:= acch4^[j-1] + 0.3597*(ch4em^[j] + ch4add*(j-50) + addmp*addch4^[j])
                         - ch4decay*(acch4^[j-1]-ch4pre);
             acn2o^[j]:= acn2o^[j-1] + 0.2079*(n2oem^[j] + n2oadd*(j-50) + addmp*addn2o^[j])
                         - n2odecay*(acn2o^[j-1]-n2opre);
          END
          ELSE BEGIN
               acch4^[j]:= acch4^[j-1] + 0.3597*(ch4em^[j] + addch4^[j])
                         - ch4decay*(acch4^[j-1]-ch4pre);
               acn2o^[j]:= acn2o^[j-1] + 0.2079*(n2oem^[j] + addn2o^[j])
                         - n2odecay*(acn2o^[j-1]-n2opre);
          END;
          rfCO2:= 6.3*log(acco2^[j]/co2pre);
          rfCH4:= 0.036*(sqrt(acCH4^[j])-sqrt(ch4pre))
                       - interact(acch4^[j],n2opre) + CH4N2O;
          rfN2O:= 0.14*(sqrt(acN2O^[j])-sqrt(N2Opre))
                       - interact(ch4pre,acn2o^[j]) + CH4N2O;

          radforc:= rfCO2 + rfCH4 + rfN2O;
          dt:= delaytemp*temps*radforc - delaytemp*temp;
          temp:= dt + temp;
(*          IF scengmt THEN BEGIN
             temp:= scentemp^[j];
             dt:= scentemp^[j] - scentemp^[j-1]
          END; *)
          tsq:= quad(temp);
          dtsq:= quad(dt);
          ds:= delaysea*seas*temp - delaysea*sea;
          sea:= ds + sea;
          ssq:= quad(sea);
          dssq:= quad(ds);
          IF j = 40+10*optper THEN
             toptper:= temp
     END
END;

PROCEDURE DrawRandom;

VAR i, j: word;
    hulp, fake, immsum: double;

BEGIN
    WITH climate DO BEGIN
         GenNormal(0,1,ch4add,n2oadd);
         ch4add:= ch4add*ch4sd;
         n2oadd:= n2oadd*n2osd;

         GenGamma(8.1270,0.3508,temps); (*mode 2.5; var 1*)
(*         GenGamma(4.9218,0.6375,temps); (*mode 2.5; var 2*)
(*         GenGamma(4.27,1.16,temps);       (*mode 3.8; sd 2.4*)
         temps:= temps/6.3/ln(2);
(*         GenGamma(6.3584,0.0317,seas); (* mode 0.17; var 0.08^2 *)
(*         GenGamma(4.0083,0.0565,seas); (* mode 0.17; var 2*0.08^2 *)
(*         GenGamma(7.4729,0.0263,seas);      (* mode 0.17; sd 0.07 *)
         GenGamma(5.9959077,0.0612581,seas); (* mode 0.30604; var 0.15^2 *)
         GenTriang(8,16,1/ch4dec,hulp);
         ch4decay:= 1/hulp;
         GenTriang(100,170,1/n2odec,hulp);
         n2odecay:= 1/hulp;
         GenTriang(0.5*lifetemp,2*lifetemp,lifetemp,hulp);
         delaytemp:= 1/hulp;
         GenTriang(0.5*lifesea,2*lifesea,lifesea,hulp);
         delaysea:= 1/hulp;
         GenNormGTZ(lifeco2,0.25*lifeco2,hulp,fake);
         decay[2]:= exp(-1/hulp);
         GenNormGTZ(lifeco3,0.25*lifeco3,hulp,fake);
         decay[3]:= exp(-1/hulp);
         GenNormGTZ(lifeco4,0.25*lifeco4,hulp,fake);
         decay[4]:= exp(-1/hulp);
         GenNormGTZ(lifeco5,0.25*lifeco5,hulp,fake);
         decay[5]:= exp(-1/hulp);
    END;
    FOR i:= 1 TO NoReg DO
        WITH economy[i] DO BEGIN
             gennormal(0,1,pgadd,ecgradd);
             pgadd:= pgadd*pgsd;
             ecgradd:= ecgradd*ecgrsd;
             gennormal(0,1,aeeiadd,aceiadd);
             aeeiadd:= aeeiadd*aeeisd;
             aceiadd:= aceiadd*aceisd;
             gennormal(0,1,foremadd,urbanadd);
             foremadd:= foremsd*foremadd;
             urbanadd:= urbansd*urbanadd;

             gennormGTZ(1,0,spbm,vosl);
             spbm:= spbmbg + spbmsd*spbm;
             vosl:= voslbg + voslsd*vosl;
             gennormGTZ(1,0,valinc,fake);
             valinc:= valincbg + valincsd*valinc;
             valadj:= valbase/valinc/(1+valbase/valinc);

             gennormGTZ(1,0.5,malel,malnl);
             gennormGTZ(1,0.5,dfel,dfnl);
             gennormGTZ(1,0.5,smel,smnl);
             gennormal(0,1,malbm,dfbm);
             gennormal(0,1,smbm,vbiadd);
             malel:= malelbg + malelsd*malel;
             malnl:= malnlbg + malnlsd*malnl;
             dfel:= dfelbg + dfelsd*dfel;
             dfnl:= dfnlbg + dfnlsd*dfnl;
             smel:= smelbg + smelsd*smel;
             smnl:= smnlbg + smnlsd*smnl;
             malbm:= malbmbg + malbmsd*malbm;
             dfbm:= dfbmbg + dfbmsd*dfbm;
             smbm:= smbmbg + smbmsd*smbm;
             vbiadd:= vbiaddbg + vbiaddsd*vbiadd;

             gennormal(0,1,chplbm,chmlbm);
             gennormal(0,1,chpqbm,chmqbm);
             gennormal(0,1,ccplbm,ccmlbm);
             gennormal(0,1,ccpqbm,ccmqbm);
             gennormal(0,1,rlbm,rqbm);
             chplbm:= ccplbmbg + chplbmsd*chplbm;
             chmlbm:= chmlbmbg + chmlbmsd*chmlbm;
             chpqbm:= chpqbmbg + chpqbmsd*chpqbm;
             chmqbm:= chmqbmbg + chmqbmsd*chmqbm;
             ccplbm:= ccplbmbg + ccplbmsd*ccplbm;
             ccmlbm:= ccmlbmbg + ccmlbmsd*ccmlbm;
             ccpqbm:= ccpqbmbg + ccpqbmsd*ccpqbm;
             ccmqbm:= ccmqbmbg + ccmqbmsd*ccpqbm;
             rlbm:= rlbmbg + rlbmsd*rlbm;
             rqbm:= rqbmbg + rqbmsd*rqbm;
             gennormGTZ(0,1,cvlin,rlin);
             gennormGTZ(0,1,plusel,maxcardvasc);
             cvlin:= cvlinbg + cvlinsd*cvlin;
             rlin:= rlinbg + rlinsd*rlin;
             plusel:= pluselbg + pluselsd*plusel;
             maxcardvasc:= maxcardvascbg + maxcardvascsd*maxcardvasc;

             gennormal(0,1,wrbm,forbm);
             gennormGTZ(0,1,wrel,wrnl);
             gennormGTZ(0,1,forel,fornl);
             gennormal(0,1,hebm,cebm);
             gennormGTZ(0,1,heel,henl);
             gennormGTZ(0,1,ceel,cenl);
             wrbm:= wrbmbg + wrbmsd*wrbm;
             forbm:= forbmbg + forbmsd*forbm;
             wrel:= wrelbg + wrelsd*wrel;
             wrnl:= wrnlbg + wrnlsd*wrnl;
             forel:= 0.31 + 0.2*forel;
             fornl:= 1 + 0.5*fornl;
             hebm:= hebmbg + hebmsd*hebm;
             cebm:= cebmbg + cebmsd*cebm;
             heel:= heelbg + heelsd*heel;
             henl:= henlbg + henlsd*henl;
             ceel:= ceelbg + ceelsd*ceel;
             cenl:= cenlbg + cenlsd*cenl;

             gennormGTZ(0,1,agrbm,agtime);
             gennormGTZ(0,1,agnl,agel);
             gennormal(0,1,aglbm,agopt);
             agrbm:= agrbmbg + agrbmsd*agrbm;
             agtime:= agtimebg + agtimesd*agtime;
             agdel:= 1 - 1/agtime;
             aglbm:= aglbmbg + aglbmsd*aglbm;
             agopt:= agoptbg + agoptsd*agopt;
             agnl:= agnlbg + agnlsd*agnl;
             agel:= agelbg + agelsd*agel;

             gennormal(0,1,wvbm,dvbm);
             wvbm:= wvbmbg + wvbmsd*wvbm;
             dvbm:= dvbmbg + dvbmsd*dvbm;

             gennormal(0,1,wlbm,wmbm);
             wlbm:= wlbmbg + wlbmsd*wlbm;
             wmbm:= wmbmbg + wmbmsd*wmbm;

             gennormal(0,1,pc,dlbm);
             pc:= pcbg + pcsd*pc;
             dlbm:= dlbmbg + dlbmsd*dlbm;

             genexponential(wetdelsd,wetdel);
             genexponential(drydelsd,drydel);
             genexponential(leavedelsd,leavedel);

             FOR j:= 1 TO NoReg DO
                 genexponential(immigratesd[j],immigrate[j]);
             gennormGTZ(0,1,immigrate[i],entertime);
             immigrate[i]:= immigratebg[i] + immigratesd[i]*immigrate[i];
             immsum:= 0;
             FOR j:= 1 TO NoReg DO
                 immsum:= immsum + immigrate[j];
             FOR j:= 1 TO NoReg DO
                 immigrate[j]:= immigrate[j]/immsum;

             entertime:= entertimebg + entertimesd*entertime;
             enterdel:= 1 - 1/entertime;

             gennormGTZ(0,1,emcst,immcst);
             emcst:= emcstbg + emcstsd*emcst;
             immcst:= immcstbg + immcstsd*immcst;

             gengamma(5.8284,0.2071,taxmp);
             gengamma(5.8284,0.2071,formp);
(*             gengamma(5.8284,0.2071,cregmp);  (* var = 0.25, mode = 1 *)
(*             gengamma(9.8990,0.1124,taxmp);
             gengamma(9.8990,0.1124,formp);
             gengamma(9.8990,0.1124,cregmp);   (* var = 0.125, mode = 1 *)
        END
END;

PROCEDURE RunEconomy;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, run : word;
    worldemm, oecdemm, ann1emm, maxkant, loss, totloss: double;

BEGIN
(*write('.');*)
     IF uncertain THEN BEGIN
        randseed:= 34514325;
        FOR run:= 1 TO runs DO BEGIN
(*gotoxy(3,23);*)write(run:5);
            DrawRandom;
            ReInit;
            FOR i:= 1 TO NYear DO BEGIN
                totalpop:= 0;
                FOR j:= 1 TO NoReg DO
                    Calculate(economy[j],i);
                IF i = 49+10*optper THEN
                   FOR j:= 1 TO NoReg DO BEGIN
                       emmrec[j]:= emmrec[j]*(run-1)/run + economy[j].emission/run;
                       IF economy[j].emission > ncemmres[j,optper] THEN
                          count[j]:= count[j]+1
                   END;
                CalcClimate(i)
            END;
            FOR i:= 1 TO NoReg DO
                economy[i].NPVW:= economy[i].hNPVW/run +
                                     (run-1)*economy[i].NPVW/run
        END
     END
     ELSE BEGIN
          ReInit;
          FOR i:= 1 TO NYear DO BEGIN
              totalpop:= 0;
              maxkant:= 0;
              totloss:= 0;
              FOR j:= 1 TO NoReg DO BEGIN
                  Calculate(economy[j],i);
                  loss:= (economy[j].eloss+economy[j].sloss);
                  totloss:= totloss + loss*economy[j].income;
                  IF loss > maxkant THEN
                     maxkant:= loss
              END;
              IF kantian THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          kant:= maxkant - (eloss+sloss);
                          kant:= kant*income
                       END;
              IF compens THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          income:= income*(1+eloss+sloss) - totloss*cummemm/globcum;
                          IF income < 0.0000001 THEN
                             income:= 0
                     END;
              IF i = 49+10*optper THEN BEGIN
                 worldemm:= 0;
                 oecdemm:= 0;
                 FOR j:= 1 TO NoReg DO BEGIN
                     IF j < 4 THEN
                        oecdemm:= oecdemm + economy[j].emission;
                     worldemm:= worldemm + economy[j].emission (*+
                                economy[j].foremp*economy[j].forestemm^[i] -
                                economy[j].forestry[trunc(i/10)]*);
                     emmrec[j]:= economy[j].emission
                 END;
                 ann1emm:= oecdemm + economy[4].emission (*- economy[2].emission*);
                 worldemm:= worldemm (*- economy[5].emission - economy[6].emission - economy[9].emission*);
                 globviol:= (worldemm > cemmres[optper])
              END;
              CalcClimate(i)
          END;
          C2200:= climate.acco2^[NYear];
          FOR i:= 1 TO NoReg DO
(*              IF C2200 > CMax THEN
                 economy[i].NPVW:= economy[i].hNPVW*
                                   (1-0.5*(noper-optper)/noper*(C2200-CMax)*(C2200-CMax)/(CBaU-CMax)/(CBaU-CMax))
              ELSE *)
                  economy[i].NPVW:= economy[i].hNPVW
     END
END;

PROCEDURE WriteEMF;

VAR i, j, k, run : word;
    oecd,eefsu,noecd,clim:text;
    TCEm, TCFE, TInc, TCCC, TERC: double;

BEGIN
     ReInit;
     assign(oecd,sessionpathname+'oecd.txt');
     rewrite(oecd);
     assign(eefsu,sessionpathname+'eefsu.txt');
     rewrite(eefsu);
     assign(noecd,sessionpathname+'noecd.txt');
     rewrite(noecd);
     assign(clim,sessionpathname+'climate.txt');
     rewrite(clim);
     FOR i:= 1 TO NYear DO BEGIN
         totalpop:= 0;
         FOR j:= 1 TO NoReg DO
             Calculate(economy[j],i);
         CalcClimate(i);
         IF (i mod 10 = 0) AND (i > 49) THEN BEGIN
            TCEm:= 0;
            TCFE:= 0;
            TInc:= 0;
            TCCC:= 0;
            TERC:= 0;
            FOR j:= 1 TO 3 DO
                WITH economy[j] DO BEGIN
                     TCEm:= TCEm + emission/1000;
                     TCFE:= TCFE + forestemm^[i] - forestry[trunc(i/10)];
                     TInc:= TInc + income/1000;
                     TCCC:= TCCC + (eloss+sloss)*income/100;
                     TERC:= TERC + 0.01*ryg*income + formp*forcost*y
                END;
            writeln(oecd,TCEm:12:6,' ',TCFE:12:6,' ',TInc:12:6,' ',TCCC:12:6,' ',TERC:12:6);
            WITH economy[4] DO
writeln(eefsu,emission/1000:12:6,' ',forestemm^[i]:12:6,' ',income/1000:12:6);
            TCEm:= 0;
            TCFE:= 0;
            TInc:= 0;
            TCCC:= 0;
            TERC:= 0;
            FOR j:= 5 TO NoReg DO
                WITH economy[j] DO BEGIN
                     TCEm:= TCEm + emission/1000;
                     TCFE:= TCFE + forestemm^[i] - forestry[trunc(i/10)];
                     TInc:= TInc + income/1000;
                     TCCC:= TCCC + (eloss+sloss)*income/100;
                     TERC:= TERC + 0.01*ryg*income + formp*forcost*y
                END;
            writeln(noecd,TCEm:12:6,' ',TCFE:12:6,' ',TInc:12:6,' ',TCCC:12:6,' ',TERC:12:6);
            WITH Climate DO BEGIN
                 write(clim,i+1950:4,' ',acco2^[i]:10:6,' ',acch4^[i]:10:6,' ',acn2o^[i]:10:6,' ',temp:10:6,' ');
                 writeln(clim,sea:12:6)
            END
         END;
     END;
     close(oecd);
     close(eefsu);
     close(noecd);
     close(clim)
END;

PROCEDURE WriteTol;

VAR i, j, k, run : word;
    econ,clim:text;
    TCEm, TCFE, TInc, TCCC, TERC: double;

BEGIN
     ReInit;
     assign(econ,sessionpathname+'tolecon.txt');
     rewrite(econ);
     assign(clim,sessionpathname+'tolclim.txt');
     rewrite(clim);
     FOR j:= 1 TO NoReg DO
         WITH economy[j] DO BEGIN
              TCFE:= forestemm^[0];
              TERC:= 0.01*ryg*income + forcost;
              IF (income > 0.0000001) AND (population > 0.0000001) THEN BEGIN
               write(econ,1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
                 write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
                 writeln(econ,TERC:12:6,' ',terc/income*100:12:6,' ',name)
              END
              ELSE BEGIN
               write(econ,1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
                   write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
                   writeln(econ,TERC:12:6,' ',0,' ',name)
              END
         END;
     WITH Climate DO BEGIN
          writeln(clim,1950:4,' ',acco2^[20]:12:6,' ',temp:12:6,' ',sea:8:6);
     END;
     FOR i:= 1 TO NYear DO BEGIN
         totalpop:= 0;
         FOR j:= 1 TO NoReg DO
             Calculate(economy[j],i);
         CalcClimate(i);
         IF (i=40) OR (i=50) OR (i=75) OR (i=100) OR (i=150) OR (i=250) THEN BEGIN
            FOR j:= 1 TO NoReg DO
                WITH economy[j] DO BEGIN
                     TCFE:= forestemm^[i] - forestry[trunc(i/10)];
                     TInc:= TInc + income;
                     TERC:= 0.01*ryg*income + formp*forcost*y;
IF income > 0.00000001 THEN BEGIN
write(econ,i+1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
writeln(econ,TERC:12:6,' ',terc/income*100:12:6,' ',name)
END
ELSE BEGIN
write(econ,i+1950:4,' ',emission:12:6,' ',TCFE:12:6,' ',income:12:6,' ',population:12:6,' ',population/pop0:10:6);
write(econ,income/population*1000:14:6,' ',eloss*100:12:6,' ',sloss*100:12:6,' ');
writeln(econ,TERC:12:6,' ',0,' ',name)
END
                END;
            WITH Climate DO
                 writeln(clim,i+1950:4,' ',acco2^[i]:12:6,' ',temp:12:6,' ',sea:12:6);
         END;
     END;
     close(econ);
     close(clim)
END;

PROCEDURE RunEconomy1;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,eap,sap,ent,mal,sch,den,car,res,lea,inc,wel,pop,clim,mc,dam:text;
    wunc,cunc,tunc,punc,gunc,iunc,eunc,runc: text;
    totpop,totinc,totimp,totemm,totred: double;
    hp,hy,hw: regarr;
    totdam, toty, loss, totloss, maxkant: double;

BEGIN
     IF uncertain THEN BEGIN
        randseed:= 34514325;
        assign(wunc,sessionpathname+'welfunc.txt');
        rewrite(wunc);
        assign(cunc,sessionpathname+'concunc.txt');
        rewrite(cunc);
        assign(tunc,sessionpathname+'tempunc.txt');
        rewrite(tunc);
        assign(punc,sessionpathname+'popunc.txt');
        rewrite(punc);
        assign(gunc,sessionpathname+'incunc.txt');
        rewrite(gunc);
        assign(iunc,sessionpathname+'impunc.txt');
        rewrite(iunc);
        assign(eunc,sessionpathname+'emmunc.txt');
        rewrite(eunc);
        assign(runc,sessionpathname+'redunc.txt');
        rewrite(runc);
        FOR run:= 1 TO runs DO BEGIN
writeln(run:5);
            DrawRandom;
            ReInit;
            FOR i:= 1 TO NYear DO BEGIN
                totalpop:= 0;
                FOR j:= 1 TO NoReg DO
                    Calculate(economy[j],i);
                CalcClimate(i);
                IF (i>=50) AND (i mod 25 = 0) THEN BEGIN
                   write(tunc,climate.temp:8:4,' ');
                   totpop:= 0;
                   totinc:= 0;
                   totimp:= 0;
                   totemm:= 0;
                   totred:= 0;
                   FOR k:= 1 TO NoReg DO BEGIN
                       totpop:= totpop + economy[k].population;
                       totinc:= totinc + economy[k].income;
                       totemm:= totemm + economy[k].emission;
                       totimp:= totimp + (economy[k].eloss+economy[k].sloss)*economy[k].income;
                       totred:= totred + economy[k].taxmp*economy[k].ryg*economy[k].income;
                   END;
                   write(punc,totpop:10:4,' ');
                   write(gunc,totinc:10:4,' ');
                   write(eunc,totemm:10:4,' ');
                   write(iunc,totimp/totinc*100:8:4,' ');
                   write(runc,totred/totinc*100:10:8,' ')
                END
            END;
            writeln(tunc);
            writeln(punc);
            writeln(gunc);
            writeln(eunc);
            writeln(iunc);
            writeln(runc);
            FOR i:= 1 TO NoReg DO
                WITH Economy[i] DO BEGIN
                     NPVW:= hNPVW/run + (run-1)*NPVW/run;
                     write(wunc,hNPVW:20:4,' ')
                END;
            writeln(wunc);
            WITH climate DO BEGIN
                 c2200:= acco2^[NYear]/run + (run-1)*c2200/run;
                 c2175:= acco2^[NYear-25]/run + (run-1)*c2175/run;
                 c2150:= acco2^[NYear-50]/run + (run-1)*c2150/run;
                 c2100:= acco2^[NYear-75]/run + (run-1)*c2125/run;
                 c2100:= acco2^[NYear-100]/run + (run-1)*c2100/run;
                 c2075:= acco2^[NYear-125]/run + (run-1)*c2075/run;
                 c2050:= acco2^[NYear-150]/run + (run-1)*c2050/run;
                 c2025:= acco2^[NYear-175]/run + (run-1)*c2025/run;
                 c2000:= acco2^[NYear-200]/run + (run-1)*c2000/run;
                 write(cunc,acco2^[NYear-200]:10:4,' ',acco2^[NYear-175]:10:4,' ');
                 write(cunc,acco2^[NYear-150]:10:4,' ',acco2^[NYear-125]:10:4,' ');
                 write(cunc,acco2^[NYear-100]:10:4,' ',acco2^[NYear-75]:10:4,' ');
                 write(cunc,acco2^[NYear-50]:10:4,' ',acco2^[NYear-25]:10:4,' ');
                 writeln(cunc,acco2^[NYear]:10:4)
            END
        END;
        close(wunc);
        close(cunc);
        close(tunc);
        close(punc);
        close(eunc);
        close(gunc);
        close(iunc);
        close(runc);
        FOR i:= 1 TO NoReg DO
            WITH Economy[i] DO BEGIN
                 p:= hp[i];
                 w1950:= hw[i];
                 y:= hy[i]
            END
     END
     ELSE BEGIN
          ReInit;
          assign(els,sessionpathname+'eloss.txt');
          rewrite(els);
          assign(sls,sessionpathname+'sloss.txt');
          rewrite(sls);
          assign(ent,sessionpathname+'enter.txt');
          rewrite(ent);
          assign(mal,sessionpathname+'malaria.txt');
          rewrite(mal);
          assign(sch,sessionpathname+'schisto.txt');
          rewrite(sch);
          assign(den,sessionpathname+'dengue.txt');
          rewrite(den);
          assign(car,sessionpathname+'cardio.txt');
          rewrite(car);
          assign(res,sessionpathname+'respir.txt');
          rewrite(res);
          assign(lea,sessionpathname+'leave.txt');
          rewrite(lea);
          assign(inc,sessionpathname+'income.txt');
          rewrite(inc);
          assign(wel,sessionpathname+'emissions.txt');
          rewrite(wel);
          assign(pop,sessionpathname+'population.txt');
          rewrite(pop);
          assign(clim,sessionpathname+'climate.txt');
           rewrite(clim);
          assign(mc,sessionpathname+'margcost.txt');
          rewrite(mc);
          assign(dam,sessionpathname+'damage.txt');
          rewrite(dam);
          FOR i:= 1 TO NYear DO BEGIN
              totalpop:= 0;
              write(els,i+1950,' ');
              write(sls,i+1950,' ');
              write(ent,i+1950,' ');
              write(mal,i+1950,' ');
              write(sch,i+1950,' ');
              write(den,i+1950,' ');
              write(car,i+1950,' ');
              write(res,i+1950,' ');
              write(lea,i+1950,' ');
              write(inc,i+1950,' ');
              write(wel,i+1950,' ');
              write(pop,i+1950,' ');
              write(mc,i+1950,' ');
              write(dam,i+1950,' ');
              totdam:= 0;
              toty:= 0;
              maxkant:= 0;
              FOR j:= 1 TO NoReg DO BEGIN
                  Calculate(economy[j],i);
                  loss:= (economy[j].eloss+economy[j].sloss);
                  IF loss > maxkant THEN
                     maxkant:= loss;
                  Write1(economy[j],i,els,sls,ent,mal,sch,den,car,res,lea,inc,wel,pop,mc);
                  totdam:= totdam + (economy[j].eloss+economy[j].sloss)*economy[j].income;
                  toty:= toty + economy[j].income
              END;
              totloss:= totdam;
              IF kantian THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          kant:= maxkant - (eloss+sloss);
                          kant:= kant*income
                     END;
              IF compens THEN
                 FOR j:= 1 TO NoReg DO
                     WITH economy[j] DO BEGIN
                          income:= income*(1+eloss+sloss)-totloss*cummemm/globcum;
                          IF income < 0.0000001 THEN
                             income:= 0
                     END;
              CalcClimate(i);
              WITH climate DO
                   writeln(clim,i+1950:4,' ',acco2^[i]:10:6,' ',acch4^[i]:10:6,' ',acn2o^[i]:10:6,' ',temp:10:6,' ',sea:8:6);
              writeln(els);
              writeln(sls);
              writeln(ent);
              writeln(mal);
              writeln(sch);
              writeln(den);
              writeln(res);
              writeln(car);
              writeln(lea);
              writeln(inc);
              writeln(wel);
              writeln(pop);
              writeln(mc);
              writeln(dam,climate.temp:8:6,' ',totdam:12:4,' ',toty:12:4,' ',totdam/toty:10:6);
          END;
          close(els);
          close(sls);
          close(ent);
          close(mal);
          close(sch);
          close(den);
          close(car);
          close(res);
          close(lea);
          close(inc);
          close(wel);
          close(pop);
          close(mc);
          close(clim);
          close(dam);
          (*WriteEMF;*)
          FOR i:= 1 TO NoReg DO
              economy[i].NPVW:= economy[i].hNPVW
     END
END;

PROCEDURE MargRun1;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,inc,pop:text;
    totpop,totinc,totimp,totemm,totred: double;
    hp,hy,hw: regarr;

BEGIN
     ReInit;
     assign(els,sessionpathname+'eloss.txt');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.txt');
     rewrite(sls);
     assign(inc,sessionpathname+'income.txt');
     rewrite(inc);
     assign(pop,sessionpathname+'population.txt');
     rewrite(pop);
     FOR i:= 1 TO NYear DO BEGIN
         totalpop:= 0;
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             WriteM(economy[j],i,els,sls,inc,pop);
         END;
         CalcClimate(i);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop);
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop);
END;

PROCEDURE MargRun2;
(* calculates the economic and climatic variables for the entire period *)

VAR i, j, k, run : word;
    els,sls,inc,pop:text;
    totpop,totinc,totimp,totemm,totred: double;
    hp,hy,hw: regarr;

BEGIN
     ReInit;
     assign(els,sessionpathname+'eloss.out');
     rewrite(els);
     assign(sls,sessionpathname+'sloss.out');
     rewrite(sls);
     assign(inc,sessionpathname+'income.out');
     rewrite(inc);
     assign(pop,sessionpathname+'population.out');
     rewrite(pop);
     FOR i:= 1 TO NYear DO BEGIN
         totalpop:= 0;
         write(els,i+1950,' ');
         write(sls,i+1950,' ');
         write(inc,i+1950,' ');
         write(pop,i+1950,' ');
         FOR j:= 1 TO NoReg DO BEGIN
             Calculate(economy[j],i);
             WriteM(economy[j],i,els,sls,inc,pop);
         END;
         CalcClimate(i);
         writeln(els);
         writeln(sls);
         writeln(inc);
         writeln(pop);
     END;
     close(els);
     close(sls);
     close(inc);
     close(pop);
END;

PROCEDURE ShortRep(name: string);

VAR i: word;
    outf: text;

BEGIN
     assign(outf,sessionpathname+name);
     rewrite(outf);
     WriteLn(outf,'     tax   reg    forestry  region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,tax[optper]:8:4,' ',forestry[optper]:8:2,'  ',name);
     WriteLn(outf);
     WriteLn(outf,'            NPVW        p        y        w       v    region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,NPVW:16:4,' ',p:8:4,' ',y:8:4,' ',w1950:8:4,' ',v:8:4,' ',name);
     WriteLn(outf);
     WriteLn(outf,'  income 90   captrans  region');
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO
              WriteLn(outf,inc0:11:6,' ',captrans:10:6,'  ',name);
     WriteLn(outf);
     FOR i:= 28 TO NYear DO
         IF ((i+1970) MOD 25) = 0 THEN
            Writeln(outf,climate.acco2^[i]:12:6);
     close(outf)
END;

FUNCTION Objective(psnr: word): double;
(* calculates the sum total of the regional NPWs *)

VAR j: word;
    objw: double;

BEGIN
     IF ineqave = 1 THEN BEGIN
        objw:= 1;
        FOR j:= 1 TO NoReg DO
            objw:= objw*economy[j].npvw
     END
     ELSE BEGIN
          objw:= 0;
          FOR j:= 1 TO psnr DO
              objw:= objw + pow(economy[j].npvw,1-ineqave);
          objw:= objw/(1-ineqave)
     END;
     objective:= objw
END;

(* OPTIMISING PROCEDURES *)

PROCEDURE CoopCBAction(VAR T,F: measure);
(* finds the optimal emission restriction by taxing (OECD) or energy and
   carbon efficiency (non-OECD) and forestry measures under cooperation *)

VAR i, rc, mc: word;
    obj0, hobj, mobj: double;

BEGIN
     RunEconomy;
     obj0:= objective(noreg);
     mobj:= -10000000;
     FOR i:= 1 TO NoReg DO BEGIN
         objtax(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= i;
            mc:= 1;
         END;
         taxobj(i);

(*         objfor(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) THEN BEGIN
             mobj:= hobj;
             rc:= i;
             mc:= 3;
         END;
         forobj(i) *)
     END;
     WHILE mobj > obj0 DO BEGIN
 (*          IF mc=1 THEN *)
              objtax(rc);
(*           IF mc=3 THEN
              objfor(rc); *)
           RunEconomy;
           obj0:= objective(noreg);
           mobj:= -10000000;
           FOR i:= 1 TO NoReg DO BEGIN
               objtax(i);
               RunEconomy;
               hobj:= objective(noreg);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 1;
               END;
               taxobj(i);

(*               objfor(i);
               RunEconomy;
               hobj:= objective(noreg);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 3;
               END;
               forobj(i) *)
           END
     END;
     FOR i:= 1 TO NoReg DO BEGIN
         T[i]:= economy[i].tax[optper];
         F[i]:= 0 (*economy[i].forestry[optper]*)
     END
END;

PROCEDURE CoopCEAction(psnr: word; VAR T,F: measure);

VAR i, j, rc, mc: word;
    obj0, hobj, mobj: double;

BEGIN

(*economy[2].tax[optper]:= europe[optper];*)

     RunEconomy;
     obj0:= objective(psnr);
(*write(obj0:8:4,' ');*)
     mobj:= -10000000;

         (*objtax(1);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 1;
            mc:= 1;
         END;
         taxobj(1);*)

     FOR i:= 1 TO psnr DO BEGIN
(*IF emmrec[i] > 0.9*ncemmres[i,optper] THEN BEGIN*)
         objtax(i);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= i;
            mc:= 1;
         END;
         taxobj(i);
(*END;*)
(*         objfor(i);
         RunEconomy;
         hobj:= objective;
         IF (hobj > mobj) THEN BEGIN
             mobj:= hobj;
             rc:= i;
             mc:= 3;
         END;
         forobj(i) *)
     END;

(*         objtax(7);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 7;
            mc:= 1;
         END;
         taxobj(7);
         objtax(8);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 8;
            mc:= 1;
         END;
         taxobj(8);*)


     WHILE (*(mobj > obj0) OR*) globviol DO BEGIN
           (*IF mc=1 THEN*)
              objtax(rc);
           (*IF mc=3 THEN
              objfor(rc);*)
           RunEconomy;
           obj0:= objective(psnr);
(*write(obj0:8:4,' ');*)
           mobj:= -10000000;

              (* objtax(1);
               RunEconomy;
               hobj:= objective(psnr);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= 1;
                  mc:= 1;
               END;
               taxobj(1);*)

           FOR i:= 1 TO psnr DO BEGIN
(*IF emmrec[i] > 0.9*ncemmres[i,optper] THEN BEGIN*)
               objtax(i);
               RunEconomy;
               hobj:= objective(psnr);
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 1;
               END;
               taxobj(i);
(*END;*)
(*               objfor(i);
               RunEconomy;
               hobj:= objective;
               IF (hobj > mobj) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
                  mc:= 3;
               END;
               forobj(i)*)
           END;
(*         objtax(7);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 7;
            mc:= 1;
         END;
         taxobj(7);
         objtax(8);
         RunEconomy;
         hobj:= objective(psnr);
         IF (hobj > mobj) THEN BEGIN
            mobj:= hobj;
            rc:= 8;
            mc:= 1;
         END;
         taxobj(8);*)


     END;
     FOR i:= 1 TO NoReg DO BEGIN
         T[i]:= economy[i].tax[optper];
         F[i]:= 0(*economy[i].forestry[optper]*)
     END
END;

PROCEDURE NonCoopCEAction(VAR T,F: measure);

VAR i, j: word;
    emm0, tobj, fobj, obj0: double;

BEGIN
     FOR i:= 1 TO NoReg DO BEGIN
write(economy[i].name,' ');
         RunEconomy;
(*writeln(ncemmres[i,optper]:8:4,' ',emmrec[i]:8:4,' ',economy[i].tax[optper]:6:4);*)
         (*obj0:= economy[i].npvw;
         emm0:= emmrec[i];
         objtax(i);
         count[i]:= 0;
         RunEconomy;
         tobj:= economy[i].npvw;
         taxobj(i);
(*         objfor(i);
         RunEconomy;
         fobj:= economy[i].npvw;
         forobj(i);   *)
         WHILE emmrec[i] > (*1.1*)ncemmres[i,optper] (*) (count[i] > 0.5*runs) (*OR (tobj > obj0) OR (fobj > obj0)*) DO BEGIN
              (* IF tobj > fobj THEN*)
                  objtax(i)
              (* ELSE
                   objfor(i)*);
(*writeln(ncemmres[i,optper]:8:4,' ',emmrec[i]:8:4,' ',economy[i].tax[optper]:6:4);*)
               RunEconomy;
               (*obj0:= economy[i].npvw;
               emm0:= emmrec[i];
               objtax(i);
               count[i]:= 0;
               RunEconomy;
               tobj:= economy[i].npvw;
               taxobj(i);
            (*   objfor(i);
               RunEconomy;
               fobj:= economy[i].npvw;
               forobj(i)  *)
         END;
         T[i]:= economy[i].tax[optper];
         F[i]:= 0 (*economy[i].forestry[optper]*)
     END
END;

PROCEDURE CoopCBA(VAR T,F: measure);

VAR i,j,k: word;

BEGIN
     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     CoopCBAction(T,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     CoopCBAction(T,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= btaxst;
     forstep:= bforst;
     CoopCBAction(T,F)
END;

PROCEDURE CoopCEA(psnr: word; VAR T,F: measure);

VAR i,j,k: word;

BEGIN
     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     CoopCEAction(psnr,T,F);
     FOR i:= 1 TO psnr DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     CoopCEAction(psnr,T,F);
     FOR i:= 1 TO psnr DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= btaxst;
     forstep:= bforst;
     CoopCEAction(psnr,T,F)
END;

PROCEDURE NonCoopCEA(VAR T,F: measure);

VAR i,j,k: word;

BEGIN
     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     NonCoopCEAction(T,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     NonCoopCEAction(T,F);
     FOR i:= 1 TO NoReg DO BEGIN
         IF economy[i].tax[optper] > 0 THEN
            taxobj(i);
         IF economy[i].forestry[optper] > 0 THEN
            forobj(i)
     END;
     taxstep:= btaxst;
     forstep:= bforst;
     NonCoopCEAction(T,F)
END;

PROCEDURE NCOptimisation(W: integer; VAR T,F: double);
(* finds the optimal emission restriction by taxing and forestry under
   non-cooperation *)

VAR objt, objf, obj0: double;
    i: word;

BEGIN
write(' ',C2200:8:2,' ',economy[w].name);

     taxstep:= 16*btaxst;
     forstep:= 16*bforst;
     RunEconomy;
(*   obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     obj0:= economy[w].npvw*(1-coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
        (*     objfor(W);
     RunEconomy;
     objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
     forobj(W); *)
     objtax(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objt:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
     taxobj(W);
     WHILE ((objt > obj0) (*OR (objf > obj0)*)) DO BEGIN
          (* IF objt > objf THEN BEGIN *)
              objtax(W);
         (*  END
           ELSE BEGIN
                objfor(W);
                RunEconomy
           END; *)
(*           obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
             obj0:= economy[w].npvw*(1-coop[w,optper]);
             FOR i:= 1 TO NoReg DO
                  obj0:= obj0 + economy[i].npvw*coop[i,optper]*coop[w,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*           objfor(W);
           RunEconomy;
           objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
           forobj(W); *)
           objtax(W);
           RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           objt:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               objt:= objt + economy[i].npvw*coop[i,optper]*coop[w,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
           taxobj(W)
     END;
     IF economy[w].tax[optper] > 0 THEN
        taxobj(w);
(*     IF economy[w].forestry[optper] > 0 THEN
        forobj(w); *)

     taxstep:= 4*btaxst;
     forstep:= 4*bforst;
     RunEconomy;
(*   obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     obj0:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         obj0:= obj0 + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*     objfor(W);
     RunEconomy;
     objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
     forobj(W); *)
     objtax(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objt:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objt:= objt + economy[i].npvw*coop[i,optper];
     taxobj(W);
     WHILE ((objt > obj0) (*OR (objf > obj0)*)) DO BEGIN
(*           IF objt > objf THEN BEGIN *)
              objtax(W);
(*           END
           ELSE BEGIN
                objfor(W);
                RunEconomy
           END;           *)
(*           obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           obj0:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               obj0:= obj0 + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*           objfor(W);
           RunEconomy;
           objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
           forobj(W); *)
           objtax(W);
           RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           objt:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               objt:= objt + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
           taxobj(W)
     END;
     IF economy[w].tax[optper] > 0 THEN
        taxobj(w);
(*     IF economy[w].forestry[optper] > 0 THEN
        forobj(w); *)

     taxstep:= btaxst;
     forstep:= bforst;
     RunEconomy;
(*   obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     obj0:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         obj0:= obj0 + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*     objfor(W);
     RunEconomy;
     objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
     forobj(W); *)
     objtax(W);
     RunEconomy;
(*     objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
     objt:= economy[w].npvw*(1 - coop[w,optper]);
     FOR i:= 1 TO NoReg DO
         objt:= objt + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
     taxobj(W);
     WHILE ((objt > obj0) (*OR (objf > obj0)*)) DO BEGIN
(*           IF objt > objf THEN BEGIN *)
              objtax(W);
(*           END
           ELSE BEGIN
                objfor(W);
                RunEconomy
           END; *)
(*           obj0:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           obj0:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               obj0:= obj0 + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        obj0:= obj0 + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
(*           objfor(W);
           RunEconomy;
           objf:= (1-altruism)*economy[w].npvw + altruism*objective(noreg);
           forobj(W); *)
           objtax(W);
           RunEconomy;
(*           objt:= (1-altruism)*economy[w].npvw + altruism*objective(noreg); *)
           objt:= economy[w].npvw*(1 - coop[w,optper]);
           FOR i:= 1 TO NoReg DO
               objt:= objt + economy[i].npvw*coop[i,optper];
     IF w <= 3 THEN
        objt:= objt + altruism*(economy[6].npvw+economy[7].npvw+economy[8].npvw+economy[9].npvw);
           taxobj(W)
     END;
     T:= economy[w].tax[optper];
     F:= 0 (*economy[w].forestry[optper]*)
END;

PROCEDURE Optimal(T,F: measure);

var i: word;

BEGIN
     SetTax4(T);
     SetFor4(F);
     RunEconomy1;
     ShortRep('summary.txt')
END;

(* FUND PROCEDURES *)

PROCEDURE NCRevRec(VAR trans: matmeas);

VAR i: word;
    h, h0: double;

BEGIN
(*write('offer ');*)
     FOR i:= 1 TO NoReg DO BEGIN
         WITH economy[i], climate DO BEGIN
              h0:= npvw;
              objtax(i);
              RunEconomy;
              h:= h0 - npvw;
              WHILE h < 0 DO BEGIN
(*write(name,' add taxes ');*)
                    h0:= npvw;
                    objtax(i);
                    RunEconomy;
                    h:= h0 - npvw
              END;
              taxobj(i);
              h:= inc0*(expo(h/88.62596)-1);
              trans[i,1]:= h;
              trans[i,2]:= -1;
              trans[i,3]:= -1;

              objfor(i);
              RunEconomy;
              h:= h0 - npvw;
              WHILE h < 0 DO BEGIN
(*write(name,' add for ');*)
                    h0:= npvw;
                    objfor(i);
                    RunEconomy;
                    h:= h0 - npvw
              END;
              forobj(i);
              h:= inc0*(expo(h/88.62596)-1);
              trans[i,3]:= h
         END;
     RunEconomy
     END
END;

FUNCTION NCRevInv(kaptr: double; k,l: word; VAR outf: text): boolean;

VAR i,m: word;
    w: ARRAY [1..NoReg] OF double;
    max: double;

BEGIN
     FOR i:= 1 TO NoReg DO BEGIN
         WITH economy[i], climate DO BEGIN
              IF kaptr < 0.25*inc0 THEN BEGIN
                 captrans:= captrans - kaptr;
                 economy[k].captrans:= economy[k].captrans + kaptr;
                 CASE l OF
                      1: objtax(k);
                      3: objfor(k)
                 END;
                 w[i]:= npvw;
                 RunEconomy;
                 w[i]:= npvw - w[i];
                 captrans:= captrans + kaptr;
                 economy[k].captrans:= economy[k].captrans - kaptr;
                 CASE l OF
                      1: taxobj(k);
                      3: forobj(k)
                 END;
                 RunEconomy
              END
              ELSE
                  w[i]:= -10000
         END
     END;
     max:= w[1];
     m:= 1;
     FOR i:= 2 TO NoReg DO
         IF w[i] > max THEN BEGIN
            m:= i;
            max:= w[i]
         END;
     IF w[m] > 0 THEN BEGIN
(*write(' from ',economy[m].name,' to ',economy[k].name);
writeLn(outf,'from ',economy[m].name,' to ',economy[k].name,' for ',l:1,' a ',kaptr);*)
        economy[m].captrans:= economy[m].captrans - kaptr;
        economy[k].captrans:= economy[k].captrans + kaptr;
        CASE l OF
             1: objtax(k);
             3: objfor(k)
        END;
        RunEconomy;
        ncrevinv:= true
     END
     ELSE
         ncrevinv:= false
END;

PROCEDURE NCFund;

VAR trans: matmeas;
    l, k, i, j: word;
    cont: boolean;
    outf: text;

BEGIN
     cont:= true;
     assign(outf,sessionpathname+'captrans.txt');
     rewrite(outf);
     WHILE cont=true DO BEGIN
           NCRevRec(trans);
           cont:= false;
(*write('accept ');*)
           FOR i:= 1 TO NoReg DO
               FOR j:= 1 TO NOI DO
                   IF trans[i,j] >= 0 THEN BEGIN
                      cont:= NCRevInv(trans[i,j],i,j,outf) OR cont
                   END
     END;
     close(outf);
     RunEconomy1;
     shortrep('fundsumm.txt');
END;

FUNCTION IdentWinLoos(VAR h: measure): boolean;

VAR i: word;
    hinv, hrec: double;
    bh: ARRAY [1..NoReg] OF boolean;

BEGIN
     FOR i:= 1 TO NoReg DO
         WITH economy[i] DO BEGIN
              h[i]:= npvw - basewf; (*npvw0[i];*)
              bh[i]:= (h[i] > 0);
              h[i]:= inc0*(expo(h[i]/88.62596)-1)
         END;
     hinv:= 0;
     hrec:= 0;
     FOR i:= 1 TO NoReg DO
         IF bh[i] THEN
(*            IF h[i] < 0.005*economy[i].inc0 THEN *)
               hinv:= hinv + h[i]
(*            ELSE BEGIN
                h[i]:= 0.005*economy[i].inc0;
                hinv:= hinv + h[i]
            END*)
         ELSE
             hrec:= hrec + h[i];
     IF hinv >= abs(hrec) THEN BEGIN
        FOR i:= 1 TO NoReg DO
            IF bh[i] THEN
               h[i]:= h[i]*hrec/hinv
            ELSE
                h[i]:= - h[i];
        IdentWinLoos:= true
     END
     ELSE
         IdentWinLoos:= false;
END;

PROCEDURE COptimiseTaxFor(VAR T,F: measure);

VAR i, j, rc, mc: word;
    obj0, hobj, mobj: double;
    kt, hkt: measure;

BEGIN
     FOR i:= 1 TO NoReg DO BEGIN
         hkt[i]:= 0;
         kt[i]:= 0
     END;
     RunEconomy;
     obj0:= objective(noreg);
(*write(obj0:8:4,' ');*)
     mobj:= -10000000;
     FOR i:= 1 TO NoReg DO BEGIN
         objtax(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) AND IdentWinLoos(hkt) THEN BEGIN
            mobj:= hobj;
            rc:= i;
(*write('tax ',economy[i].name,' ',mobj:8:4);*)

            mc:= 1;
            FOR j:= 1 TO NoReg DO
                kt[j]:= hkt[j];
         END;
         taxobj(i);

         objfor(i);
         RunEconomy;
         hobj:= objective(noreg);
         IF (hobj > mobj) AND IdentWinLoos(hkt) THEN BEGIN
            mobj:= hobj;
            rc:= i;
(*write('for ',economy[i].name,' ');*)
            mc:= 3;
            FOR j:= 1 TO NoReg DO
                kt[j]:= hkt[j];
         END;
         forobj(i);
     END;
     WHILE mobj > obj0 DO BEGIN
           obj0:= mobj;
(*write(obj0:8:4);*)
           mobj:= -10000000;
           IF mc=1 THEN
              objtax(rc);
           IF mc=3 THEN
              objfor(rc);
           FOR j:= 1 TO NoReg DO BEGIN
               economy[j].captrans:= economy[j].captrans + kt[j];
               kt[j]:= 0;
               hkt[j]:= 0
           END;
           FOR i:= 1 TO NoReg DO BEGIN
               objtax(i);
               RunEconomy;
               hobj:= objective(noreg);
               IF (hobj > mobj) AND IdentWinLoos(hkt) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
(*write('tax ',economy[i].name,' ',mobj:8:4);*)
                  mc:= 1;
                  FOR j:= 1 TO NoReg DO
                      kt[j]:= hkt[j];
               END;
               taxobj(i);

               objfor(i);
               RunEconomy;
               hobj:= objective(noreg);
               IF (hobj > mobj) AND IdentWinLoos(hkt) THEN BEGIN
                  mobj:= hobj;
                  rc:= i;
(*write('for ',economy[i].name,' ');*)
                  mc:= 3;
                  FOR j:= 1 TO NoReg DO
                      kt[j]:= hkt[j];
               END;
               forobj(i);
           END;
     END;

     FOR i:= 1 TO NoReg DO BEGIN
         T[i]:= economy[i].tax[optper];
         F[i]:= economy[i].forestry[optper]
     END
END;

PROCEDURE COptTaxFor(VAR T,F: measure);

VAR i: word;

BEGIN
     RunEconomy;
     FOR i:= 1 TO NoReg DO
         economy[i].basewf:= economy[i].npvw;
     COptimiseTaxFor(T,F);
END;

PROCEDURE CFUND;

VAR T,F: measure;

BEGIN
     WriteLn;
     COptTaxFor(T,F);
     Optimal(T,F)
END;


PROCEDURE FUND;

BEGIN
     IF cooperate THEN
        CFund
     ELSE
         NCFund
END;

(* EXIT PROCEDURE *)

PROCEDURE Exit;

VAR i: word;

BEGIN
     FOR i:= 1 TO NoReg DO
         WITH Economy[i] DO BEGIN
              dispose(ypcgrowth);
              dispose(pgrowth);
              dispose(urbanp);
              dispose(aeei);
              dispose(acei);
              dispose(forestemm)
         END;
     WITH Climate DO BEGIN
          dispose(addco2);
          dispose(addch4);
          dispose(addn2o);
          dispose(ch4em);
          dispose(n2oem);
          dispose(acco2);
          dispose(acch4);
          dispose(acn2o)
     END
END;

(* MAIN PROCEDURES *)

PROCEDURE RunNonCoopCBA;

VAR T,F: measure;
    w, cnt: integer;
    outs: text;
    stop: boolean;
    s: string;
    seet, seef: double;

BEGIN
     SetMeasures1;
     IF sequent THEN BEGIN
        assign(outs,sessionpathname+'seq0.txt');
        rewrite(outs);
        FOR optper:= startper TO NoPer DO BEGIN
            write(optper:2,' ');
            FOR w:= 1 TO NoReg DO BEGIN
                NCOptimisation(w,T[w],F[w]);
                economy[w].tax[optper]:= 0
            END;
            FOR w:= 1 TO NoReg DO
                write(outs,T[w]:6:4,' ');
            writeln(outs);
            FOR w:= 1 TO NoReg DO
                write(outs,F[w]:6:2,' ');
            writeln(outs);
        END;
        close(outs);
        cnt:= 0;
        REPEAT
              inc(cnt);
              str(cnt,s);
              assign(outs,sessionpathname+'seq'+s+'.txt');
              rewrite(outs);
              SetMeasures3(cnt-1);
              FOR optper:= StartPer TO NoPer DO BEGIN
                  write(optper:2,' ');
                  FOR w:= 1 TO NoReg DO
                      WITH economy[w] DO BEGIN
                           seet:= tax[optper];
                           seef:= forestry[optper];
                           tax[optper]:= 0;
                           forestry[optper]:= 0;
                           NCOptimisation(w,T[w],F[w]);
                           tax[optper]:= seet;
                           forestry[optper]:= seef;
                      END;
                  FOR w:= 1 TO NoReg DO
                      write(outs,((*0.9*economy[w].tax[optper]+0.1*)T[w]):6:4,' ');
                  writeln(outs);
                  FOR w:= 1 TO NoReg DO
                      write(outs,((*0.9*economy[w].forestry[optper]+0.1*)F[w]):6:2,' ');
                  writeln(outs);
              END;
              close(outs);
              stop:= checkstop(cnt)
        UNTIL (stop OR (cnt = itmax))
     END
     ELSE BEGIN
          optper:= 0;
          FOR w:= 1 TO NoReg DO
              NCOptimisation(w,T[w],F[w]);
          Optimal(T,F);
          IF captrans THEN
             NCFund
     END
END;

PROCEDURE RunNonCoopCEA;

VAR T,F: measure;
    i, cnt: word;
    s: string;
    ceaout: text;

BEGIN
     SetMeasures1;
     assign(ceaout,sessionpathname+'seq0.txt');
     rewrite(ceaout);
     FOR optper:= StartPer TO NoPer DO BEGIN
         write(optper:2,' ');
         RunEconomy;
         NonCoopCEA(T,F);
         FOR i:= 1 TO NoReg DO
             write(ceaout,T[i]:6:4,' ');
         writeln(ceaout);
         FOR i:= 1 TO NoReg DO
             write(ceaout,F[i]:6:2,' ');
         writeln(ceaout)
     END;
     close(ceaout);
(*     cnt:= 0;
     REPEAT
           inc(cnt);
           str(cnt,s);
           assign(ceaout,sessionpathname+'seq'+s+'.txt');
           rewrite(ceaout);
           SetMeasures3(cnt-1);
           FOR optper:= StartPer TO NoPer DO BEGIN
               write(optper:2,' ');
               RunEconomy;
               NonCoopCEA(T,F);
               FOR i:= 1 TO NoReg DO
                   write(ceaout,T[i]:6:4,' ');
               writeln(ceaout);
               FOR i:= 1 TO NoReg DO
                   write(ceaout,F[i]:6:2,' ');
               writeln(ceaout)
           END;
           close(ceaout);
     UNTIL checkstop(cnt)*)
END;

PROCEDURE RunCoopCBA;

VAR T,F,seet,seef: measure;
    i, cnt: word;
    outs: text;
    s: string;
    stop: boolean;

BEGIN
     IF Sequent THEN BEGIN
        assign(outs,sessionpathname+'seq0.txt');
        rewrite(outs);
        FOR optper:= StartPer TO NoPer DO BEGIN
            SetMeasures1;
            write(optper:2,' ');
            RunEconomy;
            FOR i:= 1 TO NoReg DO
                economy[i].basewf:= economy[i].npvw;
            CoopCBA(T,F);
            FOR i:= 1 TO NoReg DO
                write(outs,T[i]:8:6,' ');
            writeln(outs);
            FOR i:= 1 TO NoReg DO
                write(outs,F[i]:6:2,' ');
            writeln(outs);
        END;
        close(outs);
        cnt:= 0;
        REPEAT
              inc(cnt);
              str(cnt,s);
              assign(outs,sessionpathname+'seq'+s+'.txt');
              rewrite(outs);
              SetMeasures3(cnt-1);
              FOR optper:= StartPer TO NoPer DO BEGIN
                  write(optper:2,' ');
                  FOR i:= 1 TO NoReg DO BEGIN
                      seet[i]:= economy[i].tax[optper];
                      seef[i]:= economy[i].forestry[optper];
                      economy[i].tax[optper]:= 0;
                      economy[i].forestry[optper]:= 0
                  END;
                  RunEconomy;
                  FOR i:= 1 TO NoReg DO
                      economy[i].basewf:= economy[i].npvw;
                  CoopCBA(T,F);
                  FOR i:= 1 TO NoReg DO BEGIN
                      economy[i].tax[optper]:= seet[i];
                      economy[i].forestry[optper]:= seef[i]
                  END;
                  FOR i:= 1 TO NoReg DO
                      write(outs,(0.75*economy[i].tax[optper]+0.25*T[i]):8:6,' ');
                  writeln(outs);
                  FOR i:= 1 TO NoReg DO
                      write(outs,(0.75*economy[i].forestry[optper]+0.25*F[i]):6:2,' ');
                  writeln(outs);
              END;
              close(outs);
              stop:= checkstop(cnt)
        UNTIL (stop OR (cnt = itmax))
     END
     ELSE BEGIN
         RunEconomy;
         FOR i:= 1 TO NoReg DO
             economy[i].basewf:= economy[i].npvw;
         CoopCBA(T,F);
         Optimal(T,F)
     END
END;

PROCEDURE RunRepCBA;

VAR T,F,seet,seef: measure;
    i, w, cnt: word;
    outs: text;
    s: string;
    stop: boolean;
    seetd, seefd: double;


BEGIN
     assign(outs,sessionpathname+'seq0.txt');
     rewrite(outs);

     FOR optper:= StartPer TO RepPer DO BEGIN
         SetMeasures1;
         write(optper:2,' ');
         RunEconomy;
         FOR i:= 1 TO NoReg DO
             economy[i].basewf:= economy[i].npvw;
         CoopCBA(T,F);
         FOR i:= 1 TO NoReg DO
             write(outs,T[i]:6:4,' ');
         writeln(outs);
         FOR i:= 1 TO NoReg DO
             write(outs,F[i]:6:2,' ');
         writeln(outs);
     END;

     SetMeasures1;
     FOR optper:= RepPer + 1 TO NoPer DO BEGIN
         write(optper:2,' ');
         FOR w:= 1 TO NoReg DO BEGIN
             NCOptimisation(w,T[w],F[w]);
             economy[w].tax[optper]:= 0
         END;
         FOR w:= 1 TO NoReg DO
             write(outs,T[w]:6:4,' ');
         writeln(outs);
         FOR w:= 1 TO NoReg DO
             write(outs,F[w]:6:2,' ');
         writeln(outs);
     END;
     close(outs);
     cnt:= 0;

     REPEAT
           inc(cnt);
           str(cnt,s);
           assign(outs,sessionpathname+'seq'+s+'.txt');
           rewrite(outs);

           SetMeasures3(cnt-1);
           FOR optper:= StartPer TO RepPer DO BEGIN
               write(optper:2,' ');
               FOR i:= 1 TO NoReg DO BEGIN
                   seet[i]:= economy[i].tax[optper];
                   seef[i]:= economy[i].forestry[optper];
                   economy[i].tax[optper]:= 0;
                   economy[i].forestry[optper]:= 0
               END;
               RunEconomy;
               FOR i:= 1 TO NoReg DO
                     economy[i].basewf:= economy[i].npvw;
               CoopCBA(T,F);
               FOR i:= 1 TO NoReg DO BEGIN
                   economy[i].tax[optper]:= seet[i];
                   economy[i].forestry[optper]:= seef[i]
               END;
               FOR i:= 1 TO NoReg DO
                   write(outs,(*0.9*economy[i].tax[optper]+0.1*)T[i]:6:4,' ');
               writeln(outs);
               FOR i:= 1 TO NoReg DO
                   write(outs,(*0.9*economy[i].forestry[optper]+0.1*)F[i]:6:2,' ');
               writeln(outs);
           END;

           FOR optper:= RepPer+1 TO NoPer DO BEGIN
               write(optper:2,' ');
               FOR w:= 1 TO NoReg DO
                   WITH economy[w] DO BEGIN
                        seetd:= tax[optper];
                        seefd:= forestry[optper];
                        tax[optper]:= 0;
                        forestry[optper]:= 0;
                        NCOptimisation(w,T[w],F[w]);
                        tax[optper]:= seetd;
                        forestry[optper]:= seefd;
                   END;
                FOR w:= 1 TO NoReg DO
                    write(outs,((*0.9*economy[w].tax[optper]+0.1*)T[w]):6:4,' ');
                writeln(outs);
                FOR w:= 1 TO NoReg DO
                    write(outs,((*0.9*economy[w].forestry[optper]+0.1*)F[w]):6:2,' ');
                writeln(outs);
           END;
           close(outs);
           stop:= checkstop(cnt)
     UNTIL (stop OR (cnt = itmax))
END;

PROCEDURE RunCoopCEA;

VAR T,F: measure;
    i,psnr: word;
    ceaout: text;

BEGIN
     SetMeasures1;
(*     RunNonCoopCEA;*)
     assign(ceaout,sessionpathname+'cea.txt');
     rewrite(ceaout);
     IF Sequent THEN BEGIN
        FOR optper:= StartPer TO NoPer DO BEGIN
            write(optper:2,' ');
            RunEconomy;
            (*IF optper < 5 THEN
               psnr:= 4
            ELSE*) psnr:= 4 (*NoReg*);
            FOR i:= 1 TO NoReg DO
                economy[i].basewf:= economy[i].npvw;
            CoopCEA(psnr,T,F);
            FOR i:= 1 TO NoReg DO
                write(ceaout,T[i]:6:4,' ');
            writeln(ceaout);
            FOR i:= 1 TO NoReg DO
                write(ceaout,F[i]:6:2,' ');
            writeln(ceaout)
        END
     END;
     close(ceaout)
END;

PROCEDURE CalcMargExt(VAR out: text);

VAR els0,els1,sls0,sls1,inc0,pop0: text;
    oldinc,oldpop,newinc,newpop,elsbs,elspt,slsbs,slspt,df0,df1,df3,disc,md: regarr;
    i,j: word;
    year,marg0,marg1,marg3,margreg0,margreg1,margreg3: double;
    margwld0,margwld1,margwld3,margeu0,margeu1,margeu3: double;
    wldpop, wldinc: double;

BEGIN
     assign(els0,sessionpathname+'eloss.txt');
     reset(els0);
     assign(els1,sessionpathname+'eloss.out');
     reset(els1);
     assign(sls0,sessionpathname+'sloss.txt');
     reset(sls0);
     assign(sls1,sessionpathname+'sloss.out');
     reset(sls1);
     assign(inc0,sessionpathname+'income.txt');
     reset(inc0);
     assign(pop0,sessionpathname+'populati.txt');
     reset(pop0);
     read(inc0,year);
     read(pop0,year);
     read(els0,year);
     read(els1,year);
     read(sls0,year);
     read(sls1,year);
     FOR i:= 1 TO NoReg DO BEGIN
         read(inc0,oldinc[i]);
         read(pop0,oldpop[i]);
         read(els0,elsbs[i]);
         read(els1,elspt[i]);
         read(sls0,slsbs[i]);
         read(sls1,slspt[i]);
         df0[i]:= 1;
         df1[i]:= 1;
         df3[i]:= 1
     END;
     marg0:= 0;
     marg1:= 0;
     marg3:= 0;
     margreg0:= 0;
     margreg1:= 0;
     margreg3:= 0;
     margwld0:= 0;
     margwld1:= 0;
     margwld3:= 0;
     margeu0:= 0;
     margeu1:= 0;
     margeu3:= 0;
     readln(inc0);
     readln(pop0);
     readln(els0);
     readln(els1);
     readln(sls0);
     readln(sls1);
     FOR i:= 2 TO NYear DO BEGIN
         read(inc0,year);
         read(pop0,year);
         read(els0,year);
         read(els1,year);
         read(sls0,year);
         read(sls1,year);
         wldpop:= 0;
         wldinc:= 0;
         FOR j:= 1 TO NoReg DO BEGIN
             read(inc0,newinc[j]);
             read(pop0,newpop[j]);
             wldpop:= wldpop + newpop[j];
             wldinc:= wldinc + newinc[j];
             read(els0,elsbs[j]);
             read(els1,elspt[j]);
             read(sls0,slsbs[j]);
             read(sls1,slspt[j])
         END;
         readln(inc0);
         readln(pop0);
         readln(els0);
         readln(els1);
         readln(sls0);
         readln(sls1);
         FOR j:= 1 TO NoReg DO BEGIN
             IF (newpop[j] < 0.00001) OR (newinc[j] < 0.00001) THEN
                disc[j]:= 0
             ELSE
                 disc[j]:= (newinc[j]/newpop[j]-oldinc[j]/oldpop[j])/(oldinc[j]/oldpop[j]);
         END;
         IF (i > 49) AND (i < 150) THEN BEGIN
            FOR j:= 1 TO NoReg DO
                md[j]:= (elspt[j]+slspt[j]-elsbs[j]-slsbs[j])*newinc[j];
            marg0:= marg0 + md[2]*df0[2];
            marg1:= marg1 + md[2]*df1[2];
            marg3:= marg3 + md[2]*df3[2];
            FOR j:= 1 TO NoReg DO BEGIN
                margreg0:= margreg0 + md[j]*df0[j];
                margreg1:= margreg1 + md[j]*df1[j];
                margreg3:= margreg3 + md[j]*df3[j]
            END;
            margwld0:= margwld0 + md[1]*df0[1]*wldinc*newpop[1]/wldpop/newinc[1];
            margwld1:= margwld1 + md[1]*df1[1]*wldinc*newpop[1]/wldpop/newinc[1];
            margwld3:= margwld3 + md[1]*df3[1]*wldinc*newpop[1]/wldpop/newinc[1];
            margeu0:= margeu0 + md[1]*df0[1]*newinc[2]*newpop[1]/newpop[2]/newinc[1];
            margeu1:= margeu1 + md[1]*df1[1]*newinc[2]*newpop[1]/newpop[2]/newinc[1];
            margeu3:= margeu3 + md[1]*df3[1]*newinc[2]*newpop[1]/newpop[2]/newinc[1];
            margwld0:= margwld0 + md[2]*df0[2];
            margwld1:= margwld1 + md[2]*df1[2];
            margwld3:= margwld3 + md[2]*df3[2];
            margeu0:= margeu0 + md[2]*df0[2];
            margeu1:= margeu1 + md[2]*df1[2];
            margeu3:= margeu3 + md[2]*df3[2];
            FOR j:= 3 TO NoReg DO
                IF newinc[j] > 0.00001 THEN BEGIN
                   margwld0:= margwld0 + md[j]*df0[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld1:= margwld1 + md[j]*df1[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld3:= margwld3 + md[j]*df3[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margeu0:= margeu0 + md[j]*df0[j]*newinc[2]*newpop[j]/newpop[2]/newinc[j];
                   margeu1:= margeu1 + md[j]*df1[j]*newinc[2]*newpop[j]/newpop[2]/newinc[j];
                   margeu3:= margeu3 + md[j]*df3[j]*newinc[2]*newpop[j]/newpop[2]/newinc[j];
                END;
            FOR j:= 1 TO NoReg DO BEGIN
                df0[j]:= df0[j]/(1.00+disc[j]);
                df1[j]:= df1[j]/(1.01+disc[j]);
                df3[j]:= df3[j]/(1.03+disc[j])
            END
         END;
         FOR j:= 1 TO NoReg DO BEGIN
             oldinc[j]:= newinc[j];
             oldpop[j]:= newpop[j]
         END
     END;
     close(els0);
     close(els1);
     close(sls0);
     close(sls1);
     close(inc0);
     close(pop0);
     write(out,marg0:12:6,' ',margreg0:12:6,' ',margwld0:12:6,' ',margeu0:12:6);
     write(out,marg1:12:6,' ',margreg1:12:6,' ',margwld1:12:6,' ',margeu1:12:6);
     writeln(out,marg3:12:6,' ',margreg3:12:6,' ',margwld3:12:6,' ',margeu3:12:6)
END;

PROCEDURE CalcMarg(VAR out, out2: text);

VAR els0,els1,sls0,sls1,inc0,pop0: text;
    oldinc,oldpop,newinc,newpop,elsbs,elspt,slsbs,slspt,df0,df1,df3,disc,md,npvmd: regarr;
    i,j: word;
    year,margreg0,margreg1,margreg3, margwld0,margwld1,margwld3: double;
    wldpop, wldinc: double;

BEGIN
     assign(els0,sessionpathname+'eloss.txt');
     reset(els0);
     assign(els1,sessionpathname+'eloss.out');
     reset(els1);
     assign(sls0,sessionpathname+'sloss.txt');
     reset(sls0);
     assign(sls1,sessionpathname+'sloss.out');
     reset(sls1);
     assign(inc0,sessionpathname+'income.txt');
     reset(inc0);
     assign(pop0,sessionpathname+'populati.txt');
     reset(pop0);
     read(inc0,year);
     read(pop0,year);
     read(els0,year);
     read(els1,year);
     read(sls0,year);
     read(sls1,year);
     FOR i:= 1 TO NoReg DO BEGIN
         read(inc0,oldinc[i]);
         read(pop0,oldpop[i]);
         read(els0,elsbs[i]);
         read(els1,elspt[i]);
         read(sls0,slsbs[i]);
         read(sls1,slspt[i]);
         df0[i]:= 1;
         df1[i]:= 1;
         df3[i]:= 1;
         npvmd[i]:= 0
     END;
     margreg0:= 0;
     margreg1:= 0;
     margreg3:= 0;
     margwld0:= 0;
     margwld1:= 0;
     margwld3:= 0;
     readln(inc0);
     readln(pop0);
     readln(els0);
     readln(els1);
     readln(sls0);
     readln(sls1);
     FOR i:= 2 TO NYear DO BEGIN
         read(inc0,year);
         read(pop0,year);
         read(els0,year);
         read(els1,year);
         read(sls0,year);
         read(sls1,year);
         wldpop:= 0;
         wldinc:= 0;
         FOR j:= 1 TO NoReg DO BEGIN
             read(inc0,newinc[j]);
             read(pop0,newpop[j]);
             wldpop:= wldpop + newpop[j];
             wldinc:= wldinc + newinc[j];
             read(els0,elsbs[j]);
             read(els1,elspt[j]);
             read(sls0,slsbs[j]);
             read(sls1,slspt[j])
         END;
         readln(inc0);
         readln(pop0);
         readln(els0);
         readln(els1);
         readln(sls0);
         readln(sls1);
         FOR j:= 1 TO NoReg DO BEGIN
             IF (newpop[j] < 0.00001) OR (newinc[j] < 0.00001) THEN
                disc[j]:= 0
             ELSE
                 disc[j]:= (newinc[j]/newpop[j]-oldinc[j]/oldpop[j])/(oldinc[j]/oldpop[j]);
         END;
         IF (i > 49) AND (i < 250) THEN BEGIN
            FOR j:= 1 TO NoReg DO BEGIN
                md[j]:= (elspt[j]+slspt[j]-elsbs[j]-slsbs[j])*newinc[j];
                npvmd[j]:= npvmd[j] + md[j]*df1[j];
                margreg0:= margreg0 + md[j]*df0[j];
                margreg1:= margreg1 + md[j]*df1[j];
                margreg3:= margreg3 + md[j]*df3[j];
                IF newinc[j] > 0.00001 THEN BEGIN
                   margwld0:= margwld0 + md[j]*df0[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld1:= margwld1 + md[j]*df1[j]*wldinc*newpop[j]/wldpop/newinc[j];
                   margwld3:= margwld3 + md[j]*df3[j]*wldinc*newpop[j]/wldpop/newinc[j];
                END;
                df0[j]:= df0[j]/(1.00+disc[j]);
                df1[j]:= df1[j]/(1.01+disc[j]);
                df3[j]:= df3[j]/(1.03+disc[j])
            END
         END;
         FOR j:= 1 TO NoReg DO BEGIN
             oldinc[j]:= newinc[j];
             oldpop[j]:= newpop[j]
         END
     END;
     close(els0);
     close(els1);
     close(sls0);
     close(sls1);
     close(inc0);
     close(pop0);
     write(out,margreg0:14:8,' ',margwld0:14:8,' ');
     write(out,margreg1:14:8,' ',margwld1:14:8,' ');
     writeln(out,margreg3:14:8,' ',margwld3:14:8);
     FOR i:= 1 TO NoReg DO
         write(out2,npvmd[i]:14:8,' ');
     writeln(out2)
END;

PROCEDURE Main;

VAR i: word;

BEGIN
     Initialise;
     ReInit;
     IF CBA THEN
        IF cooperate THEN
           RunCoopCBA
        ELSE IF repeated THEN
             RunRepCBA
        ELSE
            RunNonCoopCBA
     ELSE IF CEA THEN
          IF cooperate THEN
             RunCoopCEA
          ELSE
              RunNonCoopCEA
     ELSE BEGIN
          write(' calculating');
          IF Scratch THEN
             SetMeasures1
          ELSE
              SetMeasures4;
          RunEconomy1;
          ShortRep('summary.txt')
     END;
     Exit
END;

PROCEDURE MargMain;

VAR i: word;
    out, out2: text;

BEGIN
     Initialise;
     ReInit;
     write(' calculating');
     IF scratch THEN
        SetMeasures1
     ELSE
         SetMeasures4;
     addmp:= 0;
     MargRun1;
     addmp:= 1;
     MargRun2;
     assign(out,sessionpathname+'margunc.txt');
     rewrite(out);
     assign(out2,sessionpathname+'regmarg.txt');
     rewrite(out2);
     CalcMarg(out,out2);
     FOR i:= 1 TO Runs DO BEGIN
         ReInit;
         DrawRandom;
         addmp:= 0;
         MargRun1;
         addmp:= 1;
         MargRun2;
         CalcMarg(out,out2)
     END;
     close(out);
     close(out2);
     Exit;
END;

PROCEDURE ODAMain;

VAR i,j: word;
    out, out2: text;

BEGIN
     Initialise;
     ReInit;
     write(' calculating');
     IF scratch THEN
        SetMeasures1
     ELSE
         SetMeasures4;
     assign(out,sessionpathname+'odawrd.txt');
     rewrite(out);
     assign(out2,sessionpathname+'odareg.txt');
     rewrite(out2);
     FOR i:= 1 TO NoReg DO BEGIN
         FOR j:= 1 TO NoReg DO
             economy[j].oda:= 0;
         ReInit;
         DrawRandom;
         economy[i].oda:= 0;
         MargRun1;
         economy[i].oda:= 0.01;
         MargRun2;
         CalcMarg(out,out2)
     END;
     close(out);
     close(out2);
     Exit;
END;

PROCEDURE AltMain;

VAR p,i,r: word;
    taks: double;
    outt, outw: text;

BEGIN
     Initialise;
     assign(outt,sessionpathname+'randtax.txt');
     rewrite(outt);
     assign(outw,sessionpathname+'randwel.txt');
     rewrite(outw);
     FOR p:= StartPer TO NoPer DO BEGIN
writeln(p:3);
         optper:= p;
         FOR i:= 1 TO 1000 DO BEGIN
             SetMeasures2;
             FOR r:= 1 TO NoReg DO BEGIN
                 taks:= random/100;
                 SetTax5(r,p,taks);
                 write(outt,taks:6:4,' ',taks*taks:10:8,' ')
             END;
             reinit;
             runeconomy;
             write(outt,toptper:10:6,' ',climate.temp:10:6);
             FOR r:= 1 TO NoReg DO
                 write(outw,economy[r].npvw:8:4,' ');
             writeln(outw);
             writeln(outt)
         END
     END;
     close(outw);
     close(outt)
END;

PROCEDURE AltMain2;

VAR p,i,r: word;
    taks: double;
    outt, outw: text;

BEGIN
     Initialise;
     assign(outt,sessionpathname+'randtax.txt');
     rewrite(outt);
     assign(outw,sessionpathname+'randwel.txt');
     rewrite(outw);
     FOR r:= 1 TO NoReg DO BEGIN
writeln(r:2);
         FOR i:= 1 TO 100 DO BEGIN
write(i:3,' ');
             FOR p:= StartPer TO NoPer DO BEGIN
                 optper:= p;
                 taks:= random/100;
                 SetTax5(r,p,taks);
                 write(outt,taks:6:4,' ',taks*taks:10:8,' ');
             END;
             FOR p:= StartPer TO NoPer DO BEGIN
                 optper:= p;
                 reinit;
                 runeconomy;
                 write(outt,toptper:10:6,' ');
                 write(outw,economy[r].npvw:8:4,' ');
             END;
             writeln(outt,climate.temp:10:6);
             writeln(outw)
         END
     END;
     close(outw);
     close(outt)
END;

PROCEDURE AltMain3;

VAR p,i,r: word;
    taks: double;
    outt, outw: text;

BEGIN
     Initialise;
     assign(outt,sessionpathname+'randtax.txt');
     rewrite(outt);
     assign(outw,sessionpathname+'randwel.txt');
     rewrite(outw);
     FOR i:= 1 TO 1000 DO BEGIN
write(i:4);
         FOR p:= StartPer TO NoPer DO
             FOR r:= 1 TO NoReg DO BEGIN
                 taks:= random/100;
                 SetTax5(r,p,taks);
                 write(outt,taks:6:4,' ');
             END;
         writeln(outt);
         FOR p:= StartPer TO NoPer DO BEGIN
             optper:= p;
             reinit;
             runeconomy;
             FOR r:= 1 TO NoReg DO
                 write(outw,economy[r].npvw:8:4,' ')
         END;
         writeln(outw)
     END;
     close(outw);
     close(outt)
END;

PROCEDURE AltMain4;

VAR p,i,r: word;
    taks: double;
    outt, outw, inmax: text;
    max: ARRAY [startper..noper,1..noreg] OF double;

BEGIN
     Initialise;
     assign(inmax,sessionpathname+'max.001');
     reset(inmax);
     FOR p:= StartPer TO NoPer DO BEGIN
         FOR r:= 1 TO NoReg DO
             read(inmax,max[p,r]);
         readln(inmax)
     END;
     close(inmax);
     assign(outt,sessionpathname+'randtax.txt');
     rewrite(outt);
     assign(outw,sessionpathname+'randwel.txt');
     rewrite(outw);
     FOR i:= 1 TO 1000 DO BEGIN
write(i:4);
         FOR p:= StartPer TO NoPer DO
             FOR r:= 1 TO NoReg DO BEGIN
                 taks:= random*max[p,r];
                 SetTax5(r,p,taks);
                 write(outt,taks:6:4,' ');
             END;
         writeln(outt);
         FOR p:= StartPer TO NoPer DO BEGIN
             optper:= p;
             reinit;
             runeconomy;
             FOR r:= 1 TO NoReg DO
                 write(outw,economy[r].npvw:8:4,' ')
         END;
         writeln(outw)
     END;
     close(outw);
     close(outt)
END;

PROCEDURE MainWietze;

VAR p,i,r: word;
    taks: double;
    outt, outw: text;

BEGIN
     Initialise;
     optper:= 1;
     assign(outt,sessionpathname+'randtax.txt');
     rewrite(outt);
     assign(outw,sessionpathname+'randwel.txt');
     rewrite(outw);
     FOR i:= 1 TO 1000 DO BEGIN
write(i:4);
         FOR p:= StartPer TO NoPer DO
             FOR r:= 1 TO NoReg DO BEGIN
                 taks:= random/100;
                 SetTax5(r,p,taks);
                 write(outt,taks:6:4,' ');
             END;
         writeln(outt);
         reinit;
         runeconomy;
         FOR r:= 1 TO NoReg DO
             write(outw,economy[r].npvw:8:4,' ');
         writeln(outw)
     END;
     close(outw);
     close(outt)
END;

BEGIN
     Main
END.